# $Id$
#
# Created by Greg Landrum, July 2008
#

import os
import sys
import unittest

from rdkit import Chem, DataStructs, RDConfig
from rdkit.Avalon import pyAvalonTools

struchk_conf_path = os.path.join(RDConfig.RDDataDir, 'struchk', '')
struchk_log_path = ''
STRUCHK_INIT = '''-tm
-ta %(struchk_conf_path)scheckfgs.trn
-tm
-or
-ca %(struchk_conf_path)scheckfgs.chk
-cc
-cl 3
-cs
-cn 999
-l %(struchk_log_path)sstruchk.log''' % locals()

STRUCHK_INIT_IN_MEMORY_LOGGING = '''-tm
-ta %(struchk_conf_path)scheckfgs.trn
-tm
-or
-ca %(struchk_conf_path)scheckfgs.chk
-cc
-cl 3
-cs
-cn 999''' % locals()

#
# PubChem test molecule converted from the RCSB
#  mondo errors here.
atom_clash = """2FV9_002_B_2
  RCSB PDB01151502013D
Coordinates from PDB:2FV9:B:2 Model:1 without hydrogens
 32 32  0  0  0  0            999 V2000
   46.8220   28.7360   39.6060   C 0  0  0  0  0  0  0  0  0  0  0  0
   47.3620   28.0340   38.3430   C 0  0  0  0  0  0  0  0  0  0  0  0
   47.5920   29.0540   37.2270   C 0  0  0  0  0  0  0  0  0  0  0  0
   48.4130   28.4900   36.0770   C 0  0  0  0  0  0  0  0  0  0  0  0
   47.1640   31.0380   40.1700   C 0  0  0  0  0  0  0  0  0  0  0  0
   46.6160   27.6990   40.7140   C 0  0  0  0  0  0  0  0  0  0  0  0
   45.0330   26.1340   41.6460   C 0  0  0  0  0  0  0  0  0  0  0  0
   44.1100   26.4530   42.8300   C 0  0  0  0  0  0  0  0  0  0  0  0
   49.2550   34.1450   39.3140   C 0  0  0  0  0  0  0  0  0  0  0  0
   48.2670   33.0140   39.1740   C 0  0  0  0  0  0  0  0  0  0  0  0
   44.3710   25.0810   40.7680   C 0  0  0  0  0  0  0  0  0  0  0  0
   46.3620   26.9550   37.9090   C 0  0  0  0  0  0  0  0  0  0  0  0
   47.7210   29.8260   39.9930   N 0  0  0  0  0  0  0  0  0  0  0  0
   47.5720   27.2580   41.3530   O 0  0  0  0  0  0  0  0  0  0  0  0
   44.7760   23.9020   40.8430   O 0  0  0  0  0  0  0  0  0  0  0  0
   49.7640   36.2780   39.7670   O 0  0  0  0  0  0  0  0  0  0  0  0
   44.8290   27.0700   44.0370   C 0  0  0  0  0  0  0  0  0  0  0  0
   46.0700   26.2600   44.4160   C 0  0  0  0  0  0  0  0  0  0  0  0
   43.8840   27.1450   45.2400   C 0  0  0  0  0  0  0  0  0  0  0  0
   48.7580   35.3740   39.5170   N 0  0  0  0  0  0  0  0  0  0  0  0
   50.4620   33.9280   39.2410   O 0  0  0  0  0  0  0  0  0  0  0  0
   48.1640   32.1600   40.4390   C 0  0  0  0  0  0  0  0  0  0  0  0
   47.7340   32.9890   41.6620   C 0  0  0  0  0  0  0  0  0  0  0  0
   45.9630   31.2790   40.1090   O 0  0  0  0  0  0  0  0  0  0  0  0
   45.3280   27.3570   40.9080   N 0  0  0  0  0  0  0  0  0  0  0  0
   43.4500   25.4420   40.0020   O 0  0  0  0  0  0  0  0  0  0  0  0
   47.8720   32.1800   42.9370   C 0  0  0  0  0  0  0  0  0  0  0  0
   49.0780   32.2360   43.6910   C 0  0  0  0  0  0  0  0  0  0  0  0
   49.1970   31.5090   44.9110   C 0  0  0  0  0  0  0  0  0  0  0  0
   48.1080   30.7170   45.3760   C 0  0  0  0  0  0  0  0  0  0  0  0
   46.9120   30.6330   44.6100   C 0  0  0  0  0  0  0  0  0  0  0  0
   46.7920   31.3650   43.3900   C 0  0  0  0  0  0  0  0  0  0  0  0
  1  2  1  0  0  0  0
  1  6  1  0  0  0  0
  1 13  1  0  0  0  0
  2  3  1  0  0  0  0
  2 12  1  0  0  0  0
  3  4  1  0  0  0  0
  5 13  1  0  0  0  0
  5 22  1  0  0  0  0
  5 24  2  0  0  0  0
  6 14  2  0  0  0  0
  6 25  1  0  0  0  0
  7  8  1  0  0  0  0
  7 11  1  0  0  0  0
  7 25  1  0  0  0  0
  8 17  1  0  0  0  0
  9 10  1  0  0  0  0
  9 20  1  0  0  0  0
  9 21  2  0  0  0  0
 10 22  1  0  0  0  0
 11 15  2  0  0  0  0
 11 26  1  0  0  0  0
 16 20  1  0  0  0  0
 17 18  1  0  0  0  0
 17 19  1  0  0  0  0
 22 23  1  0  0  0  0
 23 27  1  0  0  0  0
 27 28  2  0  0  0  0
 27 32  1  0  0  0  0
 28 29  1  0  0  0  0
 29 30  2  0  0  0  0
 30 31  1  0  0  0  0
 31 32  2  0  0  0  0
M  END
>  <InstanceId>
2FV9_002_B_2

>  <ChemCompId>
002

>  <PdbId>
2FV9

>  <ChainId>
B

>  <ResidueNumber>
2

>  <InsertionCode>


>  <Model>
1

>  <AltIds>


>  <MissingHeavyAtoms>
0

>  <ObservedFormula>
C23 N3 O6

>  <Name>
N-[(2R)-2-BENZYL-4-(HYDROXYAMINO)-4-OXOBUTANOYL]-L-ISOLEUCYL-L-LEUCINE

>  <SystematicName>
(2S)-2-[[(2S,3S)-2-[[(2R)-4-(hydroxyamino)-4-oxo-2-(phenylmethyl)butanoyl]amino]-3-methyl-pentanoyl]amino]-4-methyl-pentanoic acid

>  <Synonyms>


>  <Type>
NON-POLYMER

>  <Formula>
C23 H35 N3 O6

>  <MolecularWeight>
449.541

>  <ModifiedDate>
2011-06-04

>  <Parent>


>  <OneLetterCode>


>  <SubcomponentList>


>  <AmbiguousFlag>


>  <InChI>
InChI=1S/C23H35N3O6/c1-5-15(4)20(22(29)24-18(23(30)31)11-14(2)3)25-21(28)17(13-19(27)26-32)12-16-9-7-6-8-10-16/h6-10,14-15,17-18,20,32H,5,11-13H2,1-4H3,(H,24,29)(H,25,28)(H,26,27)(H,30,31)/t15-,17+,18-,20-/m0/s1

>  <InChIKey>
MWZOULASPWUGJJ-NFBUACBFSA-N

>  <SMILES>
CC[C@H](C)[C@@H](C(=O)N[C@@H](CC(C)C)C(=O)O)NC(=O)[C@H](Cc1ccccc1)CC(=O)NO

$$$$
"""


def feq(v1, v2, tol=1e-4):
  return abs(v1 - v2) < tol


class TestCase(unittest.TestCase):

  def setUp(self):
    pass

  def test1(self):
    m1 = Chem.MolFromSmiles('c1cccnc1')
    smi = pyAvalonTools.GetCanonSmiles(m1)
    self.assertTrue(smi == 'c1ccncc1')
    smi = pyAvalonTools.GetCanonSmiles('c1cccnc1', True)
    self.assertTrue(smi == 'c1ccncc1')

  def test2(self):
    tgts = [
      'CC1=CC(=O)C=CC1=O', 'c2ccc1SC(=Nc1c2)SSC4=Nc3ccccc3S4',
      '[O-][N+](=O)c1cc(Cl)c(O)c(c1)[N+]([O-])=O', 'N=C1NC=C(S1)[N+]([O-])=O',
      'Nc3ccc2C(=O)c1ccccc1C(=O)c2c3', 'OC(=O)c1ccccc1C3=C2C=CC(=O)C(Br)=C2Oc4c3ccc(O)c4Br',
      'CN(C)C2C(=O)c1ccccc1C(=O)C=2Cl', 'Cc3ccc2C(=O)c1ccccc1C(=O)c2c3[N+]([O-])=O',
      r'C/C(=N\O)/C(/C)=N/O', 'c1ccc(cc1)P(c2ccccc2)c3ccccc3'
    ]
    with open(os.path.join(RDConfig.RDDataDir, 'NCI', 'first_200.props.sdf'), 'r') as f:
      d = f.read()
    mbs = d.split('$$$$\n')[:10]
    smis = [pyAvalonTools.GetCanonSmiles(mb, False) for mb in mbs]
    self.assertTrue(smis == tgts)
    smis = [pyAvalonTools.GetCanonSmiles(smi, True) for smi in smis]
    self.assertTrue(smis == tgts)

  def test3(self):
    bv = pyAvalonTools.GetAvalonFP(Chem.MolFromSmiles('c1ccccn1'))
    self.assertEqual(len(bv), 512)
    self.assertEqual(bv.GetNumOnBits(), 20)
    bv = pyAvalonTools.GetAvalonFP(Chem.MolFromSmiles('c1ccccc1'))
    self.assertEqual(bv.GetNumOnBits(), 8)
    bv = pyAvalonTools.GetAvalonFP(Chem.MolFromSmiles('c1nnccc1'))
    self.assertEqual(bv.GetNumOnBits(), 30)
    bv = pyAvalonTools.GetAvalonFP(Chem.MolFromSmiles('c1ncncc1'))
    self.assertEqual(bv.GetNumOnBits(), 27)

    bv = pyAvalonTools.GetAvalonFP(Chem.MolFromSmiles('c1ncncc1'), nBits=1024)
    self.assertEqual(len(bv), 1024)
    self.assertTrue(bv.GetNumOnBits() > 27)

  def test4(self):
    bv = pyAvalonTools.GetAvalonFP('c1ccccn1', True)
    self.assertEqual(bv.GetNumOnBits(), 20)
    bv = pyAvalonTools.GetAvalonFP('c1ccccc1', True)
    self.assertEqual(bv.GetNumOnBits(), 8)
    bv = pyAvalonTools.GetAvalonFP('c1nnccc1', True)
    self.assertEqual(bv.GetNumOnBits(), 30)
    bv = pyAvalonTools.GetAvalonFP('c1ncncc1', True)
    self.assertEqual(bv.GetNumOnBits(), 27)
    bv = pyAvalonTools.GetAvalonFP('c1ncncc1', True, nBits=1024)
    self.assertEqual(len(bv), 1024)
    self.assertTrue(bv.GetNumOnBits() > 27)

    bv = pyAvalonTools.GetAvalonFP(Chem.MolToMolBlock(Chem.MolFromSmiles('c1ccccn1')), False)
    self.assertEqual(len(bv), 512)
    self.assertEqual(bv.GetNumOnBits(), 20)
    bv = pyAvalonTools.GetAvalonFP(Chem.MolToMolBlock(Chem.MolFromSmiles('c1ccccc1')), False)
    self.assertEqual(bv.GetNumOnBits(), 8)

  def test4b(self):
    words = pyAvalonTools.GetAvalonFPAsWords(Chem.MolFromSmiles('c1ccccn1'))
    words2 = pyAvalonTools.GetAvalonFPAsWords(Chem.MolFromSmiles('Cc1ccccn1'))
    self.assertEqual(len(words), len(words2))
    for i, word in enumerate(words):
      self.assertEqual(word & words2[i], word)

  def test5(self):
    m = Chem.MolFromSmiles('c1ccccc1C1(CC1)N')
    pyAvalonTools.Generate2DCoords(m)
    self.assertEqual(m.GetNumConformers(), 1)
    self.assertFalse(m.GetConformer(0).Is3D())

  def test6(self):
    mb = pyAvalonTools.Generate2DCoords('c1ccccc1C1(CC1)N', True)
    m = Chem.MolFromMolBlock(mb)
    self.assertEqual(m.GetNumConformers(), 1)
    self.assertFalse(m.GetConformer(0).Is3D())

  def testGitHub1062(self):
    s0 = r'C/C=C\C'
    m1 = Chem.MolFromSmiles(s0)
    s1 = Chem.MolToSmiles(m1)
    pyAvalonTools.Generate2DCoords(m1)
    mb = Chem.MolToMolBlock(m1)
    m2 = Chem.MolFromMolBlock(mb)
    s2 = Chem.MolToSmiles(m2)
    self.assertEqual(s1, s2)

    # repeat the test with an input smiles that is not canonical
    # to verify that the implementation is not sensitive to the
    # ordering of atoms
    s0 = r'C/C=C(F)\C'
    m1 = Chem.MolFromSmiles(s0)
    s1 = Chem.MolToSmiles(m1)
    self.assertNotEqual(s1, s0)
    pyAvalonTools.Generate2DCoords(m1)
    mb = Chem.MolToMolBlock(m1)
    m2 = Chem.MolFromMolBlock(mb)
    s2 = Chem.MolToSmiles(m2)
    self.assertEqual(s1, s2)

  def testRDK151(self):
    smi = "C[C@H](F)Cl"
    m = Chem.MolFromSmiles(smi)
    temp = pyAvalonTools.GetCanonSmiles(smi, True)
    self.assertEqual(temp, smi)
    temp = pyAvalonTools.GetCanonSmiles(m)
    self.assertEqual(temp, smi)

  def testStruChk(self):
    smi_good = 'c1ccccc1C1(CC-C(C)C1)C'
    smi_bad = 'c1c(R)cccc1C1(CC-C(C)C1)C'
    r = pyAvalonTools.InitializeCheckMol(STRUCHK_INIT)
    self.assertEqual(r, 0)
    (err, fixed_mol) = pyAvalonTools.CheckMolecule(smi_good, True)
    self.assertEqual(err, 0)
    mol = Chem.MolFromSmiles(smi_good)
    (err, fixed_mol) = pyAvalonTools.CheckMolecule(mol)
    self.assertEqual(err, 0)

    (err, fixed_mol) = pyAvalonTools.CheckMoleculeString(smi_good, True)
    self.assertEqual(err, 0)
    self.assertNotEqual(fixed_mol, "")
    self.assertTrue(fixed_mol.find('M  END') > 0)

    (err, fixed_mol) = pyAvalonTools.CheckMolecule(smi_bad, False)
    self.assertNotEqual(err, 0)
    self.assertFalse(fixed_mol)

    (err, fixed_mol) = pyAvalonTools.CheckMoleculeString(smi_bad, False)
    self.assertNotEqual(err, 0)
    self.assertFalse(fixed_mol)
    pyAvalonTools.CloseCheckMolFiles()

  def testStruChkInMemoryLog(self):
    r = pyAvalonTools.InitializeCheckMol(STRUCHK_INIT_IN_MEMORY_LOGGING)
    try:
      (err, fixed_mol) = pyAvalonTools.CheckMoleculeString(atom_clash, False)
      log = pyAvalonTools.GetCheckMolLog()
      self.assertTrue("of average bond length from bond" in log)

      # make sure that the log is cleared for the next molecule
      (err, fixed_mol) = pyAvalonTools.CheckMoleculeString("c1ccccc1", True)
      log = pyAvalonTools.GetCheckMolLog()
      self.assertFalse(log)

    finally:
      pyAvalonTools.CloseCheckMolFiles()

  #   def testIsotopeBug(self):
  #     mb="""D isotope problem.mol
  #   Mrv0541 08141217122D

  #   4  3  0  0  0  0            999 V2000
  #    -3.2705    0.5304    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
  #    -2.5561    0.9429    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
  #    -1.8416    0.5304    0.0000 H   0  0  0  0  0  0  0  0  0  0  0  0
  #    -2.5561    1.7679    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
  #   1  2  1  0  0  0  0
  #   2  3  1  0  0  0  0
  #   2  4  1  0  0  0  0
  # M  ISO  1   3   2
  # M  END
  # """
  #     csmi = pyAvalonTools.GetCanonSmiles(mb,False)
  #     self.assertEqual(csmi,'[2H]C(C)C')
  #     mb="""D isotope problem.mol
  #   Mrv0541 08141217122D

  #   4  3  0  0  0  0            999 V2000
  #    -3.2705    0.5304    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
  #    -2.5561    0.9429    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
  #    -1.8416    0.5304    0.0000 H   2  0  0  0  0  0  0  0  0  0  0  0
  #    -2.5561    1.7679    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
  #   1  2  1  0  0  0  0
  #   2  3  1  0  0  0  0
  #   2  4  1  0  0  0  0
  # M  ISO  1   3   2
  # M  END
  # """
  #     csmi = pyAvalonTools.GetCanonSmiles(mb,False)
  #     self.assertEqual(csmi,'[2H]C(C)C')

  #     mb="""D isotope problem.mol
  #   Mrv0541 08141217122D

  #   4  3  0  0  0  0            999 V2000
  #    -3.2705    0.5304    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
  #    -2.5561    0.9429    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
  #    -1.8416    0.5304    0.0000 D   0  0  0  0  0  0  0  0  0  0  0  0
  #    -2.5561    1.7679    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
  #   1  2  1  0  0  0  0
  #   2  3  1  0  0  0  0
  #   2  4  1  0  0  0  0
  # M  END
  # """
  #     csmi = pyAvalonTools.GetCanonSmiles(mb,False)
  #     self.assertEqual(csmi,'[2H]C(C)C')

  #   def testChiralPBug(self):
  #     mb="""Untitled Document-1
  #   Mrv0541 08161213182D

  #   5  4  0  0  0  0            999 V2000
  #    -1.1196    1.1491    0.0000 P   0  0  2  0  0  0  0  0  0  0  0  0
  #    -0.4052    1.5616    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
  #    -1.9446    1.1491    0.0000 F   0  0  0  0  0  0  0  0  0  0  0  0
  #    -1.3332    1.9460    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
  #    -0.7071    0.4346    0.0000 Cl  0  0  0  0  0  0  0  0  0  0  0  0
  #   1  2  1  0  0  0  0
  #   1  3  1  0  0  0  0
  #   1  4  2  0  0  0  0
  #   1  5  1  1  0  0  0
  # M  END
  # """
  #     r = pyAvalonTools.InitializeCheckMol(STRUCHK_INIT)
  #     self.assertEqual(r, 0)
  #     (err, fixed_mol) = pyAvalonTools.CheckMolecule(mb, False)
  #     self.assertEqual(err, 0)
  #     self.assertTrue(fixed_mol)
  #     self.assertNotEqual(fixed_mol.GetAtomWithIdx(0).GetChiralTag(),Chem.rdchem.ChiralType.CHI_UNSPECIFIED)

  def testAvalonCountFPs(self):
    # need to go to longer bit counts to avoid collisions:
    cv1 = pyAvalonTools.GetAvalonCountFP('c1ccccc1', True, nBits=6000)
    cv2 = pyAvalonTools.GetAvalonCountFP('c1ccccc1.c1ccccc1', True, nBits=6000)
    for idx, v in cv1.GetNonzeroElements().items():
      self.assertEqual(2 * v, cv2[idx])

    cv1 = pyAvalonTools.GetAvalonCountFP(Chem.MolFromSmiles('c1ccccc1'), nBits=6000)
    cv2 = pyAvalonTools.GetAvalonCountFP(Chem.MolFromSmiles('c1ccccc1.c1ccccc1'), nBits=6000)
    for idx, v in cv1.GetNonzeroElements().items():
      self.assertEqual(2 * v, cv2[idx])


if __name__ == '__main__':
  unittest.main()



#
# Copyright (C) 2019 Greg Landrum and T5 Informatics GmbH
#  All Rights Reserved
#
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#

import pickle
import unittest

from rdkit import Chem, RDConfig, rdBase
from rdkit.Chem.Scaffolds import rdScaffoldNetwork

rdBase.DisableLog("rdApp.info")


class TestScaffoldNetwork(unittest.TestCase):
  """
  RDKit must have been built with Boost serialization support
  for this test to pass.
  """

  def test1Pickle(self):
    smis = ["c1ccccc1CC1NC(=O)CCC1", "c1cccnc1CC1NC(=O)CCC1"]
    ms = [Chem.MolFromSmiles(x) for x in smis]
    params = rdScaffoldNetwork.ScaffoldNetworkParams()
    params.includeScaffoldsWithoutAttachments = False
    net = rdScaffoldNetwork.CreateScaffoldNetwork(ms, params)
    self.assertEqual(len(net.nodes), 7)
    self.assertEqual(len(net.edges), 7)

    pkl = pickle.dumps(net)
    net2 = pickle.loads(pkl)
    self.assertEqual(len(net2.nodes), 7)
    self.assertEqual(len(net2.edges), 7)
    self.assertEqual(list(net2.nodes), list(net.nodes))
    self.assertEqual([str(x) for x in net2.edges], [str(x) for x in net.edges])


if __name__ == '__main__':
  unittest.main()



# $Id$
#
#  Copyright (C) 2007  greg Landrum
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#

import doctest
import unittest
from io import StringIO

from rdkit import Chem, RDLogger
from rdkit.VLib.NodeLib import (SDSupply, SmartsMolFilter, SmartsRemover,
                                SmilesDupeFilter, SmilesOutput, SmilesSupply)
from rdkit.VLib.Supply import SupplyNode


def load_tests(loader, tests, ignore):
  """ Add the Doctests from the module """
  tests.addTests(doctest.DocTestSuite(SDSupply, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(SmartsMolFilter, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(SmartsRemover, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(SmilesDupeFilter, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(SmilesOutput, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(SmilesSupply, optionflags=doctest.ELLIPSIS))
  #   tests.addTests(doctest.DocTestSuite(DbMolSupply, optionflags=doctest.ELLIPSIS))
  return tests


class Test_NodeLib(unittest.TestCase):

  def tearDown(self):
    RDLogger.EnableLog('rdApp.error')

  def test_SmartsMolFilter(self):
    smis = ['C1CCC1', 'C1CCC1C=O', 'CCCC', 'CCC=O', 'CC(=O)C', 'CCN', 'NCCN', 'NCC=O']
    mols = [Chem.MolFromSmiles(x) for x in smis]
    suppl = SupplyNode(contents=mols)
    self.assertEqual(len(list(suppl)), 8)

    smas = ['C=O', 'CN']
    counts = [1, 2]
    filt = SmartsMolFilter.SmartsFilter(patterns=smas, counts=counts)
    filt.AddParent(suppl)
    self.assertEqual(len(list(filt)), 5)

    suppl.reset()
    filt.SetNegate(True)
    self.assertEqual(len(list(filt)), 3)

    smas = ['C=O', 'CN']
    filt = SmartsMolFilter.SmartsFilter(patterns=smas)
    filt.AddParent(suppl)
    self.assertEqual(len(list(filt)), 6)

    self.assertRaises(ValueError, SmartsMolFilter.SmartsFilter, patterns=smas, counts=[
      'notEnough',
    ])
    RDLogger.DisableLog('rdApp.error')
    self.assertRaises(ValueError, SmartsMolFilter.SmartsFilter, patterns=['BadSmarts'])
    RDLogger.EnableLog('rdApp.error')

  def test_SmilesOutput(self):
    smis = ['C1CCC1', 'C1CC1', 'C=O', 'CCN']
    mols = [Chem.MolFromSmiles(x) for x in smis]
    for i, mol in enumerate(mols, 100):
      mol.SetProp('ID', str(i))

    suppl1 = SupplyNode(contents=mols)
    suppl2 = SupplyNode(contents='abcd')

    sio = StringIO()
    node = SmilesOutput.OutputNode(idField='ID', dest=sio, delim=', ')
    node.AddParent(suppl1)
    node.AddParent(suppl2)
    list(node)
    self.assertEqual(sio.getvalue(), '100, C1CCC1, a\n101, C1CC1, b\n102, C=O, c\n103, CCN, d\n')

  def test_SmartsRemover(self):
    salts = ['[Cl;H1&X1,-]', '[Na+]', '[O;H2,H1&-,X0&-2]', 'BadSmarts']
    RDLogger.DisableLog('rdApp.error')
    self.assertRaises(ValueError, SmartsRemover.SmartsRemover, patterns=salts)
    RDLogger.EnableLog('rdApp.error')

  def test_SmilesDupeFilter(self):
    smis = ['C1CCC1', 'CCCC', 'CCCC', 'C1CCC1']
    mols = [Chem.MolFromSmiles(x) for x in smis]
    suppl = SupplyNode(contents=mols)
    self.assertEqual(len(list(suppl)), 4)

    dupFilter = SmilesDupeFilter.DupeFilter()
    dupFilter.AddParent(suppl)
    self.assertEqual(len(list(dupFilter)), 2)


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



#  Copyright (c) 2024 David Cosgrove and other RDKit contributors
#  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials provided
#       with the distribution.
#     * Neither the name of Novartis Institutes for BioMedical Research Inc.
#       nor the names of its contributors may be used to endorse or promote
#       products derived from this software without specific prior written
#       permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

# These tests are just to check that the Python wrappers are working
# ok.  The bulk of the tests are in the C++ code.
import os
import tempfile
import unittest

from pathlib import Path

from rdkit import Chem, rdBase
from rdkit.Chem import (rdSynthonSpaceSearch, rdFingerprintGenerator,
                        rdRascalMCES, rdGeneralizedSubstruct)


class TestCase(unittest.TestCase):

  def setUp(self):
    self.sssDir = Path(os.environ["RDBASE"]) / "Code" / "GraphMol" / "SynthonSpaceSearch" / "data"

  def testSubstructSearch(self):
    fName = self.sssDir / "idorsia_toy_space_a.spc"
    synthonspace = rdSynthonSpaceSearch.SynthonSpace()
    synthonspace.ReadDBFile(fName)
    params = rdSynthonSpaceSearch.SynthonSpaceSearchParams()
    params.maxHits = 10
    results = synthonspace.SubstructureSearch(Chem.MolFromSmarts("c1ccccc1C(=O)N1CCCC1"), params=params)
    self.assertEqual(10, len(results.GetHitMolecules()))
    smParams = Chem.SubstructMatchParameters()
    results = synthonspace.SubstructureSearch(Chem.MolFromSmarts("c1ccccc1C(=O)N1CCCC1"),
                                              substructMatchParams=smParams,
                                              params=params)
    self.assertEqual(10, len(results.GetHitMolecules()))

  def testFingerprintSearch(self):
    fName = self.sssDir / "idorsia_toy_space_a.spc"
    synthonspace = rdSynthonSpaceSearch.SynthonSpace()
    synthonspace.ReadDBFile(fName)
    self.assertEqual(6, synthonspace.GetNumReactions())
    params = rdSynthonSpaceSearch.SynthonSpaceSearchParams()
    params.maxHits = -1
    params.similarityCutoff = 0.45
    fpgen = rdFingerprintGenerator.GetRDKitFPGenerator(fpSize=2048, useBondOrder=True)
    results = synthonspace.FingerprintSearch(
      Chem.MolFromSmiles("O=C(Nc1c(CNC=O)cc[s]1)c1nccnc1"), fpgen, params)
    self.assertEqual(278, len(results.GetHitMolecules()))
    

  def testEnumerate(self):
    fName = self.sssDir / "amide_space.txt"
    synthonspace = rdSynthonSpaceSearch.SynthonSpace()
    synthonspace.ReadTextFile(fName)
    with tempfile.NamedTemporaryFile() as tmp:
      synthonspace.WriteEnumeratedFile(tmp.name)

  def testTimeOut(self):
    fName = self.sssDir / "idorsia_toy_space_a.spc"
    synthonspace = rdSynthonSpaceSearch.SynthonSpace()
    synthonspace.ReadDBFile(fName)
    params = rdSynthonSpaceSearch.SynthonSpaceSearchParams()
    params.timeOut = 1
    params.maxHits = -1
    params.similarityCutoff = 0.3
    params.fragSimilarityAdjuster = 0.3
    fpgen = rdFingerprintGenerator.GetRDKitFPGenerator(fpSize=2048, useBondOrder=True)
    results = synthonspace.FingerprintSearch(
      Chem.MolFromSmiles("c12ccc(C)cc1[nH]nc2C(=O)NCc1cncs1"), fpgen, params)
    self.assertTrue(results.GetTimedOut())

  def testSynthonError(self):
    fName = self.sssDir / "amide_space_error.txt"
    synthonspace = rdSynthonSpaceSearch.SynthonSpace()
    self.assertRaises(RuntimeError, synthonspace.ReadTextFile, fName)

    fName = self.sssDir / "synthon_error.txt"
    synthonspace = rdSynthonSpaceSearch.SynthonSpace()
    self.assertRaises(RuntimeError, synthonspace.ReadTextFile, fName)

  def testRascalSearch(self):
    fName = self.sssDir / "Syntons_5567.csv"
    synthonspace = rdSynthonSpaceSearch.SynthonSpace()
    synthonspace.ReadTextFile(fName)
    self.assertEqual(10, synthonspace.GetNumReactions())
    params = rdSynthonSpaceSearch.SynthonSpaceSearchParams()
    params.maxHits = 10
    rascalOpts = rdRascalMCES.RascalOptions()
    results = synthonspace.RascalSearch(Chem.MolFromSmiles("c12ccc(C)cc1[nH]nc2C(=O)NCc1cncs1"),
                                             rascalOpts, params)
    self.assertEqual(10, len(results.GetHitMolecules()))

  def testExtendedSubsructureSearch(self):
    fName = self.sssDir / "extended_query.csv"
    synthonspace = rdSynthonSpaceSearch.SynthonSpace()
    synthonspace.ReadTextFile(fName)
    self.assertEqual(7, synthonspace.GetNumReactions())
    m = Chem.MolFromSmarts('[#6]-*.c1nc2cccnc2n1 |m:1:3.10|')
    xqry = rdGeneralizedSubstruct.CreateExtendedQueryMol(m)
    results = synthonspace.SubstructureSearch(xqry)
    self.assertEqual(12, len(results.GetHitMolecules()))
  
    
if __name__ == "__main__":
  unittest.main()



#
#  Copyright (C) 2002-2008  greg Landrum and Rational Discovery LLC
#
""" unit testing code for molecular descriptor calculators

"""
import os.path
import pickle
import unittest
from io import BytesIO, StringIO

import numpy

from rdkit import Chem, RDConfig
from rdkit.ML.Descriptors import Descriptors, MoleculeDescriptors
from rdkit.TestRunner import redirect_stdout


class TestCase(unittest.TestCase):

  def setUp(self):
    self.descs = ['MolLogP', 'Chi1v']
    self.vers = ('1.1.0', '1.0.0')
    self.calc = MoleculeDescriptors.MolecularDescriptorCalculator(self.descs)
    self.testD = [('CCOC', (0.6527, 1.40403)), ('CC=O', (0.2052, 0.81305)),
                  ('CCC(=O)O', (0.481, 1.48839))]

  def testGetNames(self):
    self.assertEqual(self.calc.GetDescriptorNames(), tuple(self.descs))

  def _testVals(self, calc, testD):
    for smi, vals in testD:
      mol = Chem.MolFromSmiles(smi)
      ans = numpy.array(vals)
      res = numpy.array(calc.CalcDescriptors(mol))
      self.assertTrue(
        max(abs(res - ans)) < 1e-4, 'bad descriptor values for SMILES %s (%s)' % (smi, str(res)))

  def testCalcVals(self):
    self._testVals(self.calc, self.testD)

  def testSaveState(self):
    fName = os.path.join(RDConfig.RDCodeDir, 'ML/Descriptors/test_data', 'molcalc.dsc')
    with open(fName, 'r') as inTF:
      buf = inTF.read().replace('\r\n', '\n').encode('utf-8')
      inTF.close()
    inF = BytesIO(buf)
    calc = pickle.load(inF)
    self.assertEqual(calc.GetDescriptorNames(), tuple(self.descs))
    self.assertEqual(calc.GetDescriptorVersions(), tuple(self.vers))
    self._testVals(calc, self.testD)

    f = StringIO()
    with redirect_stdout(f):
      calc.ShowDescriptors()
    s = f.getvalue()
    for name in calc.GetDescriptorNames():
      self.assertIn(name, s)

    self.assertIn('Wildman-Crippen LogP value', calc.GetDescriptorSummaries())
    self.assertIn('N/A', calc.GetDescriptorSummaries())

    funcs = calc.GetDescriptorFuncs()
    self.assertEqual(len(funcs), len(self.descs))
    for f in funcs:
      self.assertTrue(callable(f))


class TestDescriptors(unittest.TestCase):

  def test_DescriptorCalculator(self):
    calc = Descriptors.DescriptorCalculator()
    self.assertRaises(NotImplementedError, calc.ShowDescriptors)
    self.assertRaises(NotImplementedError, calc.GetDescriptorNames)
    self.assertRaises(NotImplementedError, calc.CalcDescriptors, None)

    calc.simpleList = ['simple1', 'simple2']
    calc.compoundList = ['cmpd1', 'cmpd2']
    f = StringIO()
    with redirect_stdout(f):
      calc.ShowDescriptors()
    s = f.getvalue()
    for name in calc.simpleList:
      self.assertIn(name, s)
    for name in calc.compoundList:
      self.assertIn(name, s)

  def test_github3511(self):
    mol = Chem.MolFromSmiles('C')
    descriptors = [name for name, _ in Chem.Descriptors.descList]
    calculator = MoleculeDescriptors.MolecularDescriptorCalculator(descriptors)
    calculator.CalcDescriptors(mol)

    # This should not raise a pickling exception
    pickle.dumps(mol)


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



from __future__ import print_function
import os, sys
import unittest
import copy
from rdkit import DataStructs, RDConfig
from rdkit.Geometry import rdGeometry as geom
from rdkit.Chem import rdMIF, AllChem
import math
import numpy as np
import pickle


def feq(v1, v2, tol=1.0e-4):
  return abs(v1 - v2) < tol


class testfunctor():

  def __call__(self, x, y, z):
    return 1.0

  def __call__(self, pt):
    return 1.0


class TestCase(unittest.TestCase):

  def SetUp(self):
    pass

  def test1ConstructGrid(self):
    mol = AllChem.MolFromMolFile(
      os.path.join(RDConfig.RDBaseDir, 'Code/GraphMol/MolInteractionFields/Wrap/testData/HCl.mol'),
      removeHs=False)

    grd = rdMIF.ConstructGrid(mol, margin=5.0, spacing=0.5)

    bond = mol.GetConformer().GetAtomPosition(1) - mol.GetConformer().GetAtomPosition(0)
    self.assertTrue(feq(grd.GetSpacing(), 0.5))
    self.assertTrue(grd.GetNumX() == int((abs(bond.x) + 10.0) / 0.5 + 0.5))
    self.assertTrue(grd.GetNumY() == int((abs(bond.y) + 10.0) / 0.5 + 0.5))
    self.assertTrue(grd.GetNumZ() == int((abs(bond.z) + 10.0) / 0.5 + 0.5))

  def test2CubeFiles(self):
    mol = AllChem.MolFromMolFile(
      os.path.join(RDConfig.RDBaseDir, 'Code/GraphMol/MolInteractionFields/Wrap/testData/HCl.mol'),
      removeHs=False)

    grd = geom.UniformRealValueGrid3D(5.0, 5.0, 5.0, 1.0, geom.Point3D(0.0, 0.0, 0.0))
    for i in range(grd.GetSize()):
      grd.SetVal(i, float(i / 10.0))

    rdMIF.WriteToCubeFile(
      grd,
      os.path.join(RDConfig.RDBaseDir,
                   'Code/GraphMol/MolInteractionFields/Wrap/testData/test3.cube'),
      mol)

    grd2, mol2 = rdMIF.ReadFromCubeFile(
      os.path.join(RDConfig.RDBaseDir,
                   'Code/GraphMol/MolInteractionFields/Wrap/testData/test3.cube'))

    self.assertTrue(grd.GetSize() == grd2.GetSize())

    for i in range(grd.GetSize()):
      self.assertTrue(feq(grd2.GetVal(i), float(i / 10.0)))

    self.assertTrue(grd.CompareGrids(grd2))
    self.assertTrue(mol.GetNumAtoms() == mol2.GetNumAtoms())

    for i in range(mol.GetNumAtoms()):
      self.assertTrue(mol.GetAtomWithIdx(i).GetAtomicNum() == mol2.GetAtomWithIdx(i).GetAtomicNum())
      self.assertTrue(
        feq(mol.GetConformer().GetAtomPosition(i).x,
            mol2.GetConformer().GetAtomPosition(i).x))
      self.assertTrue(
        feq(mol.GetConformer().GetAtomPosition(i).y,
            mol2.GetConformer().GetAtomPosition(i).y))
      self.assertTrue(
        feq(mol.GetConformer().GetAtomPosition(i).z,
            mol2.GetConformer().GetAtomPosition(i).z))

    rdMIF.WriteToCubeFile(
      grd,
      os.path.join(RDConfig.RDBaseDir,
                   'Code/GraphMol/MolInteractionFields/Wrap/testData/test4.cube'))

    grd3, mol3 = rdMIF.ReadFromCubeFile(
      os.path.join(RDConfig.RDBaseDir,
                   'Code/GraphMol/MolInteractionFields/Wrap/testData/test4.cube'))

    self.assertTrue(grd.GetSize() == grd3.GetSize())

    for i in range(grd.GetSize()):
      self.assertTrue(feq(grd3.GetVal(i), float(i / 10.0)))

    self.assertTrue(grd.CompareGrids(grd3))
    self.assertIsNone(mol3)

  def test3Coulomb(self):
    mol = AllChem.MolFromMolFile(
      os.path.join(RDConfig.RDBaseDir, 'Code/GraphMol/MolInteractionFields/Wrap/testData/HCl.mol'),
      removeHs=False)
    AllChem.ComputeGasteigerCharges(mol)

    conf = mol.GetConformer(0)
    charges = [a.GetDoubleProp("_GasteigerCharge") for a in mol.GetAtoms()]
    pos = conf.GetPositions()

    grd = rdMIF.ConstructGrid(mol)

    grd2 = copy.deepcopy(grd)

    coul = rdMIF.Coulomb(mol)

    rdMIF.CalculateDescriptors(grd, coul)
    coul1 = rdMIF.Coulomb(charges, pos)
    chargesMismatchingLen = charges[:-1]
    self.assertRaises(ValueError, rdMIF.Coulomb, chargesMismatchingLen, pos)
    rdMIF.CalculateDescriptors(grd2, coul1)
    self.assertTrue(grd.CompareParams(grd2))
    self.assertTrue(grd.CompareVectors(grd2))
    self.assertTrue(grd.CompareGrids(grd2))
    self.assertTrue(feq(coul(0.0, 0.0, 0.0, 1000), 0.0))
    self.assertTrue(coul(2.0, 0.0, 0.0, 1000) < 0)
    self.assertTrue(coul(-2.0, 0.0, 0.0, 1000) > 0)

    rdMIF.CalculateDescriptors(grd, rdMIF.Coulomb(mol, absVal=True))

    for i in range(grd.GetSize()):
      self.assertTrue(grd.GetVal(i) <= 0.0)

    coul1 = rdMIF.Coulomb(mol, probeCharge=-1.0)
    self.assertTrue(coul1(-2.0, 0.0, 0.0, 1000) < 0)
    self.assertTrue(coul1(2.0, 0.0, 0.0, 1000) > 0)

    coul2 = rdMIF.Coulomb(mol, probeCharge=-.5)
    self.assertTrue(coul1(-2.0, 0.0, 0.0, 1000) < coul2(-2.0, 0.0, 0.0, 1000))

    coul3 = rdMIF.Coulomb(mol, confId=0, probeCharge=1.0, absVal=False,
                          chargeKey="_GasteigerCharge", softcoreParam=0.01, cutoff=1.0)
    self.assertTrue(coul3(0.0, 0.0, 0.0, 1000) > coul3(0.1, 0.0, 0.0, 1000))
    self.assertTrue(coul3(0.66, 0.0, 0.0, 1000) > coul3(0.68, 0.0, 0.0, 1000))
    self.assertTrue(coul3(0.70, 0.0, 0.0, 1000) > coul3(0.68, 0.0, 0.0, 1000))

  def test4CoulombDielectric(self):
    mol = AllChem.MolFromMolFile(
      os.path.join(RDConfig.RDBaseDir, 'Code/GraphMol/MolInteractionFields/Wrap/testData/HCl.mol'),
      removeHs=False)
    AllChem.ComputeGasteigerCharges(mol)

    conf = mol.GetConformer(0)
    charges = [a.GetDoubleProp("_GasteigerCharge") for a in mol.GetAtoms()]
    pos = conf.GetPositions()

    grd = rdMIF.ConstructGrid(mol, confId=0)
    grd2 = rdMIF.ConstructGrid(mol, confId=0)

    couldiele = rdMIF.CoulombDielectric(mol, confId=0)
    couldiele1 = rdMIF.CoulombDielectric(charges, pos)
    chargesMismatchingLen = charges[:-1]
    self.assertRaises(ValueError, rdMIF.CoulombDielectric, chargesMismatchingLen, pos)

    rdMIF.CalculateDescriptors(grd, couldiele)
    rdMIF.CalculateDescriptors(grd2, couldiele1)

    self.assertTrue(grd.CompareGrids(grd2))
    self.assertTrue(feq(couldiele(0.0, 0.0, 0.0, 1000), 0.0))
    self.assertTrue(couldiele(2.0, 0.0, 0.0, 1000) < 0)
    self.assertTrue(couldiele(-2.0, 0.0, 0.0, 1000) > 0)

    rdMIF.CalculateDescriptors(grd, rdMIF.CoulombDielectric(mol, absVal=True))

    for i in range(grd.GetSize()):
      self.assertTrue(grd.GetVal(i) <= 0.0)

    couldiele1 = rdMIF.CoulombDielectric(mol, probeCharge=-1.0)
    self.assertTrue(couldiele1(-2.0, 0.0, 0.0, 1000) < 0)
    self.assertTrue(couldiele1(2.0, 0.0, 0.0, 1000) > 0)

    couldiele2 = rdMIF.CoulombDielectric(mol, probeCharge=-.5)
    self.assertTrue(couldiele1(-2.0, 0.0, 0.0, 1000) < couldiele2(-2.0, 0.0, 0.0, 1000))

    couldiele3 = rdMIF.CoulombDielectric(mol, confId=0, probeCharge=1.0, absVal=False,
                                         chargeKey="_GasteigerCharge", softcoreParam=0.01,
                                         cutoff=1.0)
    self.assertTrue(couldiele3(0.0, 0.0, 0.0, 1000) > couldiele3(0.1, 0.0, 0.0, 1000))
    self.assertTrue(couldiele3(0.66, 0.0, 0.0, 1000) > couldiele3(0.68, 0.0, 0.0, 1000))
    self.assertTrue(couldiele3(0.70, 0.0, 0.0, 1000) > couldiele3(0.68, 0.0, 0.0, 1000))

    mol = AllChem.MolFromMolFile(
      os.path.join(RDConfig.RDBaseDir,
                   'Code/GraphMol/MolInteractionFields/Wrap/testData/glucose.mol'), removeHs=False)
    AllChem.ComputeGasteigerCharges(mol)

    couldiele4 = rdMIF.CoulombDielectric(mol, confId=0, probeCharge=1.0, absVal=False,
                                         chargeKey="_GasteigerCharge", softcoreParam=0.01,
                                         cutoff=1.0, epsilon=80.0, xi=4.0)
    couldiele5 = rdMIF.CoulombDielectric(mol, confId=0, probeCharge=1.0, absVal=False,
                                         chargeKey="_GasteigerCharge", softcoreParam=0.01,
                                         cutoff=1.0, epsilon=200.0, xi=4.0)
    couldiele6 = rdMIF.CoulombDielectric(mol, confId=0, probeCharge=1.0, absVal=False,
                                         chargeKey="_GasteigerCharge", softcoreParam=0.01,
                                         cutoff=1.0, epsilon=80.0, xi=10.0)

    self.assertTrue(couldiele5(-1.0, 0.0, 0.0, 1000) < couldiele4(-1.0, 0.0, 0.0, 1000))
    self.assertTrue(couldiele6(-1.0, 0.0, 0.0, 1000) < couldiele4(-1.0, 0.0, 0.0, 1000))

  def test5VdWaals(self):
    mol = AllChem.MolFromMolFile(
      os.path.join(RDConfig.RDBaseDir, 'Code/GraphMol/MolInteractionFields/Wrap/testData/HCN.mol'),
      removeHs=False)
    vdw = rdMIF.MMFFVdWaals(mol, confId=0, probeAtomType=6, scaling=False, cutoff=1.0)

    self.assertTrue(vdw(-5.0, 0, 0, 1000) < 0)
    self.assertTrue(vdw(-1.68, 0, 0, 1000) > vdw(-5.0, 0, 0, 1000))
    self.assertTrue(vdw(-5.0, 0, 0, 1000) < vdw(-10.0, 0, 0, 1000))

    mol2 = AllChem.MolFromMolFile(
      os.path.join(RDConfig.RDBaseDir, 'Code/GraphMol/MolInteractionFields/Wrap/testData/h2o.mol'),
      removeHs=False)
    vdw = rdMIF.MMFFVdWaals(mol2, scaling=False)
    vdw2 = rdMIF.MMFFVdWaals(mol2, scaling=True)

    self.assertTrue(abs(vdw2(-3.0, 0, 0, 1000) - vdw(-3.0, 0, 0, 1000)) > 0.0001)

    vdw3 = rdMIF.UFFVdWaals(mol, confId=0, probeAtomType="O_3", cutoff=1.0)

    self.assertTrue(vdw3(-5.0, 0, 0, 1000) < 0)
    self.assertTrue(vdw3(-1.68, 0, 0, 1000) > vdw3(-5.0, 0, 0, 1000))
    self.assertTrue(vdw3(-5.0, 0, 0, 1000) < vdw3(-10.0, 0, 0, 1000))

  def test6HBond(self):
    mol = AllChem.MolFromMolFile(
      os.path.join(RDConfig.RDBaseDir,
                   'Code/GraphMol/MolInteractionFields/Wrap/testData/ethane.mol'), removeHs=False)
    grd = rdMIF.ConstructGrid(mol, margin=5.0, spacing=0.5)
    for i in range(grd.GetSize()):
      grd.SetVal(i, 1.0)
    grd1 = copy.deepcopy(grd)

    hbonddes = rdMIF.HBond(mol)
    rdMIF.CalculateDescriptors(grd, hbonddes)
    self.assertTrue(not grd.CompareGrids(grd1))
    self.assertTrue(
      abs(int((grd.GetOccupancyVect() - grd1.GetOccupancyVect()).GetTotalVal())) == grd.GetSize())

    hbonddes = rdMIF.HBond(mol, probeAtomType='O')
    for i in range(grd.GetSize()):
      grd.SetVal(i, 1.0)
    rdMIF.CalculateDescriptors(grd, hbonddes)
    self.assertTrue(not grd.CompareGrids(grd1))
    self.assertTrue(
      abs(int((grd.GetOccupancyVect() - grd1.GetOccupancyVect()).GetTotalVal())) == grd.GetSize())

    mol = AllChem.MolFromMolFile(
      os.path.join(RDConfig.RDBaseDir,
                   'Code/GraphMol/MolInteractionFields/Wrap/testData/aceticacid.mol'),
      removeHs=False)
    grd = rdMIF.ConstructGrid(mol, margin=5.0, spacing=0.5)

    hbonddes = rdMIF.HBond(mol, probeAtomType="OH")
    rdMIF.CalculateDescriptors(grd, hbonddes)

    hbonddes1 = rdMIF.HBond(mol, probeAtomType="O", fixed=True)
    rdMIF.CalculateDescriptors(grd, hbonddes1)

    hbonddes2 = rdMIF.HBond(mol, probeAtomType="O", fixed=False)
    self.assertTrue(hbonddes1(4.0, 0.0, 1.0, 1000) > hbonddes2(4.0, 0.0, 1.0, 1000))

    hbonddes3 = rdMIF.HBond(mol, probeAtomType="NH")
    self.assertTrue(hbonddes(2.0, 2.0, 1.0, 1000) < hbonddes3(2.0, 2.0, 1.0, 1000))

    hbonddes4 = rdMIF.HBond(mol, probeAtomType="N")
    self.assertTrue(hbonddes1(3.0, 0.0, 0.0, 1000) < hbonddes4(3.0, 0.0, 0.0, 1000))

  def test7Hydrophilic(self):
    mol = AllChem.MolFromMolFile(
      os.path.join(RDConfig.RDBaseDir, 'Code/GraphMol/MolInteractionFields/Wrap/testData/h2o.mol'),
      removeHs=False)
    hydro = rdMIF.Hydrophilic(mol)
    hbondOH = rdMIF.HBond(mol, probeAtomType="OH")
    hbondO = rdMIF.HBond(mol, probeAtomType="O")

    pt = geom.Point3D(0.0, 0.0, 0.0)
    hyd = hydro(pt.x, pt.y, pt.z, 1000)
    hOH = hbondOH(pt.x, pt.y, pt.z, 1000)
    hO = hbondO(pt.x, pt.y, pt.z, 1000)
    self.assertTrue(feq(min(hOH, hO), hyd))

    pt = geom.Point3D(1.0, 1.5, 2.0)
    hyd = hydro(pt.x, pt.y, pt.z, 1000)
    hOH = hbondOH(pt.x, pt.y, pt.z, 1000)
    hO = hbondO(pt.x, pt.y, pt.z, 1000)
    self.assertTrue(feq(min(hOH, hO), hyd))

    pt = geom.Point3D(2.0, 1.5, -3.0)
    hyd = hydro(pt.x, pt.y, pt.z, 1000)
    hOH = hbondOH(pt.x, pt.y, pt.z, 1000)
    hO = hbondO(pt.x, pt.y, pt.z, 1000)
    self.assertTrue(feq(min(hOH, hO), hyd))

    pt = geom.Point3D(-2.5, 0.5, 3.0)
    hyd = hydro(pt.x, pt.y, pt.z, 1000)
    hOH = hbondOH(pt.x, pt.y, pt.z, 1000)
    hO = hbondO(pt.x, pt.y, pt.z, 1000)
    self.assertTrue(feq(min(hOH, hO), hyd))

    pt = geom.Point3D(10.0, 1.5, 1.0)
    hyd = hydro(pt.x, pt.y, pt.z, 1000)
    hOH = hbondOH(pt.x, pt.y, pt.z, 1000)
    hO = hbondO(pt.x, pt.y, pt.z, 1000)
    self.assertTrue(feq(min(hOH, hO), hyd))

    pt = geom.Point3D(6.0, -5.0, 0.0)
    hyd = hydro(pt.x, pt.y, pt.z, 1000)
    hOH = hbondOH(pt.x, pt.y, pt.z, 1000)
    hO = hbondO(pt.x, pt.y, pt.z, 1000)
    self.assertTrue(feq(min(hOH, hO), hyd))

    pt = geom.Point3D(-3.0, -3.0, 7.0)
    hyd = hydro(pt.x, pt.y, pt.z, 1000)
    hOH = hbondOH(pt.x, pt.y, pt.z, 1000)
    hO = hbondO(pt.x, pt.y, pt.z, 1000)
    self.assertTrue(feq(min(hOH, hO), hyd))

    pt = geom.Point3D(1.0, 0.0, 0.0)
    hyd = hydro(pt.x, pt.y, pt.z, 1000)
    hOH = hbondOH(pt.x, pt.y, pt.z, 1000)
    hO = hbondO(pt.x, pt.y, pt.z, 1000)
    self.assertTrue(feq(min(hOH, hO), hyd))

    pt = geom.Point3D(0.0, 2.0, 2.0)
    hyd = hydro(pt.x, pt.y, pt.z, 1000)
    hOH = hbondOH(pt.x, pt.y, pt.z, 1000)
    hO = hbondO(pt.x, pt.y, pt.z, 1000)
    self.assertTrue(feq(min(hOH, hO), hyd))

    pt = geom.Point3D(2.0, -2.0, 0.0)
    hyd = hydro(pt.x, pt.y, pt.z, 1000)
    hOH = hbondOH(pt.x, pt.y, pt.z, 1000)
    hO = hbondO(pt.x, pt.y, pt.z, 1000)
    self.assertTrue(feq(min(hOH, hO), hyd))

    pt = geom.Point3D(2.0, -2.0, -3.0)
    hyd = hydro(pt.x, pt.y, pt.z, 1000)
    hOH = hbondOH(pt.x, pt.y, pt.z, 1000)
    hO = hbondO(pt.x, pt.y, pt.z, 1000)
    self.assertTrue(feq(min(hOH, hO), hyd))


if __name__ == '__main__':
  print("Testing MIF wrapper")
  unittest.main()



#  Copyright (C) 2017-2021  Novartis Institute of BioMedical Research
#   and other RDKit contributors
#
#         All Rights Reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials provided
#       with the distribution.
#     * Neither the name of Novartis Institutes for BioMedical Research Inc.
#       nor the names of its contributors may be used to endorse or promote
#       products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
""" This is a rough coverage test of the python wrapper for the SubstructLibrary

it is intended to be shallow but broad.
"""

import doctest
import logging
import os
import sys
import unittest

from rdkit import RDConfig, RDLogger, rdBase
from rdkit.RDLogger import logger

logger = logger()
import pickle
import tempfile
import time

from rdkit import Chem
from rdkit.Chem import rdSubstructLibrary, rdGeneralizedSubstruct, rdTautomerQuery


def load_tests(loader, tests, ignore):
  tests.addTests(doctest.DocTestSuite(rdSubstructLibrary))
  return tests


def makeStereoExamples():
  el = "NO"
  mols = []
  for e in el:
    for e2 in el:
      if e != e2:
        smi = "C1CCO[C@@](%s)(%s)1" % (e, e2)
        m = Chem.MolFromSmiles(smi)
        if m:
          mols.append(m)
        smi = "C1CCO[C@](%s)(%s)1" % (e, e2)
        m = Chem.MolFromSmiles(smi)
        if m:
          mols.append(m)

  return mols


class TestCase(unittest.TestCase):

  def setUp(self):
    pass

  def test0SubstructLibrary(self):
    for keyholderCls in [None, rdSubstructLibrary.KeyFromPropHolder]:
      for fpholderCls in [None, rdSubstructLibrary.PatternHolder]:
        for holder in [
            rdSubstructLibrary.MolHolder(),
            rdSubstructLibrary.CachedMolHolder(),
            rdSubstructLibrary.CachedSmilesMolHolder()
        ]:
          if fpholderCls:
            fpholder = fpholderCls()
          else:
            fpholder = None
          if keyholderCls:
            keyholder = keyholderCls()
            self.assertEqual(keyholder.GetPropName(), "_Name")
          else:
            keyholder = None
          slib_ = rdSubstructLibrary.SubstructLibrary(holder, fpholder, keyholder)
          for i in range(100):
            m = Chem.MolFromSmiles("c1ccccc1")
            m.SetProp("_Name", str(i))
            self.assertEqual(slib_.AddMol(m), i)

          libs = [slib_]
          if rdSubstructLibrary.SubstructLibraryCanSerialize():
            serialized1 = pickle.loads(pickle.dumps(slib_))
            serialized2 = rdSubstructLibrary.SubstructLibrary(slib_.Serialize())
            libs.append(serialized1)
            libs.append(serialized2)

          for slib in libs:
            res = slib.GetMatches(m)

            if keyholderCls:
              for idx in res:
                self.assertEqual(str(idx), slib.GetKeyHolder().GetKey(idx))
              self.assertEqual([str(idx) for idx in res], list(slib.GetKeyHolder().GetKeys(res)))

            t2 = time.time()
            self.assertTrue(len(res) == 100)

            res = slib.GetMatches(m)

            self.assertEqual(len(res), 100)
            self.assertTrue(set(res) == set(list(range(100))))

            res = slib.GetMatches(m, maxResults=100)
            self.assertEqual(len(res), 100)
            self.assertEqual(len(slib.GetMatches(m, startIdx=0, endIdx=100)), 100)

            self.assertTrue(slib.HasMatch(m))
            self.assertEqual(slib.CountMatches(m), 100)

  def test1SubstructLibrary(self):
    for keyholderCls in [None, rdSubstructLibrary.KeyFromPropHolder]:
      for fpholderCls in [None, rdSubstructLibrary.PatternHolder]:
        for holder in [
            rdSubstructLibrary.MolHolder(),
            rdSubstructLibrary.CachedMolHolder(),
            rdSubstructLibrary.CachedSmilesMolHolder()
        ]:
          if fpholderCls:
            fpholder = fpholderCls()
          else:
            fpholder = None
          if keyholderCls:
            keyholder = keyholderCls()
            self.assertEqual(keyholder.GetPropName(), "_Name")
          else:
            keyholder = None

          slib_ = rdSubstructLibrary.SubstructLibrary(holder, fpholder, keyholder)
          mols = []
          for i in range(100):
            m = Chem.MolFromSmiles("c1ccccc1")
            m.SetProp("_Name", str(i * 2))
            self.assertEqual(slib_.AddMol(m), i * 2)
            mols.append(m)
            m2 = Chem.MolFromSmiles("CCCC")
            m2.SetProp("_Name", str(i * 2 + 1))
            self.assertEqual(slib_.AddMol(m2), i * 2 + 1)
            mols.append(m2)

          libs = [slib_]
          if rdSubstructLibrary.SubstructLibraryCanSerialize():
            serialized1 = pickle.loads(pickle.dumps(slib_))
            serialized2 = rdSubstructLibrary.SubstructLibrary(slib_.Serialize())
            libs.append(serialized1)
            libs.append(serialized2)

          for slib in libs:
            res = slib.GetMatches(m)
            self.assertEqual(len(res), 100)
            self.assertEqual(set(res), set(list(range(0, 200, 2))))
            if keyholderCls:
              self.assertEqual([str(idx) for idx in res], [str(idx) for idx in range(0, 200, 2)])

            res = slib.GetMatches(m2)
            self.assertEqual(len(res), 100)
            self.assertTrue(set(res) == set(list(range(1, 200, 2))))
            if keyholderCls:
              self.assertEqual([str(idx) for idx in res], [str(idx) for idx in range(1, 200, 2)])

            res = slib.GetMatches(m)
            self.assertEqual(len(res), 100)

            res = slib.GetMatches(m, maxResults=100)
            self.assertEqual(len(res), 100)

            self.assertEqual(len(slib.GetMatches(m, startIdx=0, endIdx=50 * 2)), 50)
            self.assertEqual(len(slib.GetMatches(m2, startIdx=1, endIdx=50 * 2 + 1)), 50)

            self.assertTrue(slib.HasMatch(m))
            self.assertTrue(slib.HasMatch(m2))
            self.assertEqual(slib.CountMatches(m), 100)
            self.assertEqual(slib.CountMatches(m2), 100)

  def testOptions(self):
    mols = makeStereoExamples() * 10

    for holderCls in [
        rdSubstructLibrary.MolHolder,
        rdSubstructLibrary.CachedMolHolder,
        rdSubstructLibrary.CachedSmilesMolHolder,
        rdSubstructLibrary.CachedTrustedSmilesMolHolder,
    ]:
      holder = holderCls()
      slib_ = rdSubstructLibrary.SubstructLibrary(holder, None)

      for mol in mols:
        slib_.AddMol(mol)

      libs = [slib_]
      if rdSubstructLibrary.SubstructLibraryCanSerialize():
        serialized1 = pickle.loads(pickle.dumps(slib_))
        serialized2 = rdSubstructLibrary.SubstructLibrary(slib_.Serialize())
        libs.append(serialized1)
        libs.append(serialized2)

      for slib in libs:
        core = Chem.MolFromSmarts("C-1-C-C-O-C(-*)(-*)1")
        res = slib.GetMatches(core)
        self.assertEqual(len(res),
                         len([x for x in mols if x.HasSubstructMatch(core, useChirality=True)]))

        core = Chem.MolFromSmarts("C-1-C-C-O-C(-[O])(-[N])1")
        core.SetProp("core", "core")
        res = slib.GetMatches(core, useChirality=False)
        self.assertEqual(len(res),
                         len([x for x in mols if x.HasSubstructMatch(core, useChirality=False)]))

        core = Chem.MolFromSmarts("C-1-C-C-O-[C@@](-[O])(-[N])1")
        res = slib.GetMatches(core, useChirality=False)
        self.assertEqual(len(res),
                         len([x for x in mols if x.HasSubstructMatch(core, useChirality=False)]))

        core = Chem.MolFromSmarts("C-1-C-C-O-[C@@](-[O])(-[N])1")
        res = slib.GetMatches(core)
        self.assertEqual(len(res),
                         len([x for x in mols if x.HasSubstructMatch(core, useChirality=True)]))

  def testSmilesCache(self):
    mols = makeStereoExamples() * 10
    holder = rdSubstructLibrary.CachedSmilesMolHolder()

    slib_ = rdSubstructLibrary.SubstructLibrary(holder, None)

    for mol in mols:
      holder.AddSmiles(Chem.MolToSmiles(mol, isomericSmiles=True))

    libs = [slib_]
    if rdSubstructLibrary.SubstructLibraryCanSerialize():
      serialized1 = pickle.loads(pickle.dumps(slib_))
      serialized2 = rdSubstructLibrary.SubstructLibrary(slib_.Serialize())
      libs.append(serialized1)
      libs.append(serialized2)

    for slib in libs:
      core = Chem.MolFromSmarts("C-1-C-C-O-C(-*)(-*)1")
      res = slib.GetMatches(core)
      self.assertEqual(len(res),
                       len([x for x in mols if x.HasSubstructMatch(core, useChirality=True)]))

      core = Chem.MolFromSmarts("C-1-C-C-O-C(-[O])(-[N])1")
      core.SetProp("core", "core")
      res = slib.GetMatches(core, useChirality=False)
      self.assertEqual(len(res),
                       len([x for x in mols if x.HasSubstructMatch(core, useChirality=False)]))

      core = Chem.MolFromSmarts("C-1-C-C-O-[C@@](-[O])(-[N])1")
      res = slib.GetMatches(core, useChirality=False)
      self.assertEqual(len(res),
                       len([x for x in mols if x.HasSubstructMatch(core, useChirality=False)]))

      core = Chem.MolFromSmarts("C-1-C-C-O-[C@@](-[O])(-[N])1")
      res = slib.GetMatches(core)
      self.assertEqual(len(res),
                       len([x for x in mols if x.HasSubstructMatch(core, useChirality=True)]))

  def testTrustedSmilesCache(self):
    mols = makeStereoExamples() * 10
    holder = rdSubstructLibrary.CachedTrustedSmilesMolHolder()

    slib_ = rdSubstructLibrary.SubstructLibrary(holder, None)

    for mol in mols:
      holder.AddSmiles(Chem.MolToSmiles(mol, isomericSmiles=True))

    libs = [slib_]
    if rdSubstructLibrary.SubstructLibraryCanSerialize():
      serialized1 = pickle.loads(pickle.dumps(slib_))
      serialized2 = rdSubstructLibrary.SubstructLibrary(slib_.Serialize())
      libs.append(serialized1)
      libs.append(serialized2)

    for slib in libs:
      core = Chem.MolFromSmarts("C-1-C-C-O-C(-*)(-*)1")
      res = slib.GetMatches(core)
      self.assertEqual(len(res),
                       len([x for x in mols if x.HasSubstructMatch(core, useChirality=True)]))

      core = Chem.MolFromSmarts("C-1-C-C-O-C(-[O])(-[N])1")
      core.SetProp("core", "core")
      res = slib.GetMatches(core, useChirality=False)
      self.assertEqual(len(res),
                       len([x for x in mols if x.HasSubstructMatch(core, useChirality=False)]))

      core = Chem.MolFromSmarts("C-1-C-C-O-[C@@](-[O])(-[N])1")
      res = slib.GetMatches(core, useChirality=False)
      self.assertEqual(len(res),
                       len([x for x in mols if x.HasSubstructMatch(core, useChirality=False)]))

      core = Chem.MolFromSmarts("C-1-C-C-O-[C@@](-[O])(-[N])1")
      res = slib.GetMatches(core)
      self.assertEqual(len(res),
                       len([x for x in mols if x.HasSubstructMatch(core, useChirality=True)]))

  def testBinaryCache(self):
    mols = makeStereoExamples() * 10
    holder = rdSubstructLibrary.CachedMolHolder()

    slib_ = rdSubstructLibrary.SubstructLibrary(holder, None)

    for mol in mols:
      holder.AddBinary(mol.ToBinary())

    libs = [slib_]
    if rdSubstructLibrary.SubstructLibraryCanSerialize():
      serialized1 = pickle.loads(pickle.dumps(slib_))
      serialized2 = rdSubstructLibrary.SubstructLibrary(slib_.Serialize())
      libs.append(serialized1)
      libs.append(serialized2)

    for slib in libs:
      core = Chem.MolFromSmarts("C-1-C-C-O-C(-*)(-*)1")
      res = slib.GetMatches(core)
      self.assertEqual(len(res),
                       len([x for x in mols if x.HasSubstructMatch(core, useChirality=True)]))

      core = Chem.MolFromSmarts("C-1-C-C-O-C(-[O])(-[N])1")
      core.SetProp("core", "core")
      res = slib.GetMatches(core, useChirality=False)
      self.assertEqual(len(res),
                       len([x for x in mols if x.HasSubstructMatch(core, useChirality=False)]))

      core = Chem.MolFromSmarts("C-1-C-C-O-[C@@](-[O])(-[N])1")
      res = slib.GetMatches(core, useChirality=False)
      self.assertEqual(len(res),
                       len([x for x in mols if x.HasSubstructMatch(core, useChirality=False)]))

      core = Chem.MolFromSmarts("C-1-C-C-O-[C@@](-[O])(-[N])1")
      res = slib.GetMatches(core)
      self.assertEqual(len(res),
                       len([x for x in mols if x.HasSubstructMatch(core, useChirality=True)]))

  def testRingSmartsWithTrustedSmiles(self):
    pat = Chem.MolFromSmarts("[C&R1]")
    pat2 = Chem.MolFromSmarts("C@C")  # ring bond
    holder = rdSubstructLibrary.CachedTrustedSmilesMolHolder()
    lib = rdSubstructLibrary.SubstructLibrary(holder)
    lib.AddMol(Chem.MolFromSmiles("C1CC1"))

    # make sure we can get an unsanitized molecule that fails (no ring info)
    print("Testing atom rings")
    with self.assertRaises(RuntimeError):
      holder.GetMol(0).HasSubstructMatch(pat)
    print("testing bond rings")
    with self.assertRaises(RuntimeError):
      holder.GetMol(0).HasSubstructMatch(pat2)

    # shouldn't throw
    print("searching atom rings")
    self.assertEqual(len(lib.GetMatches(pat)), 1)
    self.assertEqual(lib.CountMatches(pat), 1)
    print("searching bond rings")
    self.assertEqual(len(lib.GetMatches(pat2)), 1)
    self.assertEqual(lib.CountMatches(pat2), 1)
    print("done")

  @unittest.skipIf(not rdBase._serializationEnabled, "not built with serialization support")
  def test_init_from_and_to_stream(self):
    mols = makeStereoExamples() * 10
    holder = rdSubstructLibrary.CachedSmilesMolHolder()

    # one day I'll fix this, but we need to write text but read binary
    #  grrr....  something about the python_streambuf handler.
    slib = rdSubstructLibrary.SubstructLibrary(holder, None)

    for mol in mols:
      holder.AddSmiles(Chem.MolToSmiles(mol, isomericSmiles=True))

    if rdSubstructLibrary.SubstructLibraryCanSerialize():
      fd, path = tempfile.mkstemp()
      with open(path, 'w') as file:
        slib.ToStream(file)

      with open(path, 'rb') as file:
        slib2 = rdSubstructLibrary.SubstructLibrary()
        slib2.InitFromStream(file)
        self.assertEqual(len(slib), len(slib2))

    from io import BytesIO, StringIO
    s = StringIO()
    slib.ToStream(s)

    sb = BytesIO(s.getvalue().encode("ascii"))
    self.assertTrue(len(sb.getvalue()) > 0)
    slib3 = rdSubstructLibrary.SubstructLibrary()
    slib3.InitFromStream(sb)
    self.assertEqual(len(slib), len(slib2))

  def test_addpatterns(self):
    pdb_ligands = [
      "CCS(=O)(=O)c1ccc(OC)c(Nc2ncc(-c3cccc(-c4ccccn4)c3)o2)c1",
      "COc1ccc(S(=O)(=O)NCC2CC2)cc1Nc1ncc(-c2cccc(-c3cccnc3)c2)o1",
      "COc1ccc(-c2oc3ncnc(N)c3c2-c2ccc(NC(=O)Nc3cc(C(F)(F)F)ccc3F)cc2)cc1",
      "COC(=O)Nc1nc2ccc(Oc3ccc(NC(=O)Nc4cc(C(F)(F)F)ccc4F)cc3)cc2[nH]1",
      "COc1cc(Nc2ncnc(-c3cccnc3Nc3ccccc3)n2)cc(OC)c1OC",
      "O=C(Nc1ccc(Oc2ccccc2)cc1)c1cccnc1NCc1ccncc1", "O=C(Nc1ccc(Oc2ccccc2)cc1)c1cccnc1NCc1ccncc1",
      "CNC(=O)c1cc(Oc2ccc3[nH]c(Nc4ccc(Cl)c(C(F)(F)F)c4)nc3c2)ccn1",
      "CNC(=O)c1cc(Oc2ccc3oc(Nc4ccc(Cl)c(OCC5CCC[NH+]5C)c4)nc3c2)ccn1",
      "CNC(=O)c1cc(Oc2ccc3oc(Nc4ccc(Cl)c(OCC5CCC[NH+]5C)c4)nc3c2)ccn1",
      "COc1cc2nccc(Oc3ccc4c(c3)OCCN4C(=O)Nc3ccc(Cl)cc3)c2cc1OC",
      "CNC(=O)c1c(C)oc2cc(Oc3cc[nH+]c4cc(OCCN5CCOCC5)ccc34)ccc12",
      "COc1cc2[nH+]ccc(Oc3ccc4c(C(=O)Nc5ccc(Cl)cc5)cccc4c3)c2cc1OC",
      "COc1cc2[nH+]ccc(Oc3ccc4c(C(=O)Nc5ccc(Cl)cc5)cccc4c3)c2cc1OC",
      "COc1cc2[nH+]ccc(Oc3ccc4c(C(=O)NC5CC5)cccc4c3)c2cc1OC",
      "COc1cc2[nH+]ccc(Oc3ccc4c(C(=O)NC5CC5)cccc4c3)c2cc1OC",
      "Cc1ccc(C(=O)Nc2cc(CCC[NH+](C)C)cc(C(F)(F)F)c2)cc1Nc1ncccc1-c1ccncn1",
      "COc1cc(Nc2nccc(Nc3ccc4c(C)n[nH]c4c3)n2)cc(OC)c1OC",
      "COc1cc(Nc2nccc(N(C)c3ccc4c(C)n[nH]c4c3)n2)cc(OC)c1OC",
      "Cc1ccn(-c2ccc3c(c2)NCC3(C)C)c(=O)c1-c1ccc2nc(N)ncc2c1",
      "Cc1ccn(-c2ccc3c(c2)NCC3(C)C)c(=O)c1-c1ccc2nc(N)ncc2c1",
      "Cc1ccc(C(=O)NCCC2CCCC2)cc1C(=O)Nc1ccc(N)nc1", "Cc1ccc(C(=O)NCCC2CCCC2)cc1C(=O)Nc1ccc(N)nc1",
      "Cc1ccn(-c2cccc(C(F)(F)F)c2)c(=O)c1-c1ccc2nc(N)ncc2c1",
      "Cc1ccn(-c2cccc(C(F)(F)F)c2)c(=O)c1-c1ccc2nc(N)ncc2c1",
      "O=C(Nc1cncnc1)c1c(Cl)ccc2c(Nc3cccc(C(F)(F)F)c3)noc12",
      "O=C(Nc1cncnc1)c1c(Cl)ccc2c(Nc3cccc(C(F)(F)F)c3)noc12",
      "CC1(C)CNc2cc(NC(=O)c3cccnc3NCc3ccncc3)ccc21", "CC1(C)CNc2cc(NC(=O)c3cccnc3NCc3ccncc3)ccc21"
    ]

    for patterns in [
        rdSubstructLibrary.PatternHolder(),
        rdSubstructLibrary.TautomerPatternHolder()
    ]:
      mols = [Chem.MolFromSmiles(smi) for smi in pdb_ligands]
      holder = rdSubstructLibrary.CachedMolHolder()
      slib_with_patterns = rdSubstructLibrary.SubstructLibrary(holder, patterns)

      for mol in mols:
        slib_with_patterns.AddMol(mol)

      for nthreads in [1, 2, 0]:
        slib_without_patterns = rdSubstructLibrary.SubstructLibrary(holder, None)
        rdSubstructLibrary.AddPatterns(slib_without_patterns, nthreads)
        # check for seg fault
        #  were the fingerprints really created
        slib_without_patterns.GetFpHolder().GetFingerprint(0)
        for mol in mols:
          l1 = slib_with_patterns.CountMatches(mol)
          l2 = slib_without_patterns.CountMatches(mol)
          self.assertTrue(l1)
          self.assertEqual(l1, l2)

  def test_basic_addpatterns(self):
    # add mols
    pdb_ligands = [
      "CCS(=O)(=O)c1ccc(OC)c(Nc2ncc(-c3cccc(-c4ccccn4)c3)o2)c1",
      "COc1ccc(S(=O)(=O)NCC2CC2)cc1Nc1ncc(-c2cccc(-c3cccnc3)c2)o1",
      "COc1ccc(-c2oc3ncnc(N)c3c2-c2ccc(NC(=O)Nc3cc(C(F)(F)F)ccc3F)cc2)cc1",
      "COC(=O)Nc1nc2ccc(Oc3ccc(NC(=O)Nc4cc(C(F)(F)F)ccc4F)cc3)cc2[nH]1",
      "COc1cc(Nc2ncnc(-c3cccnc3Nc3ccccc3)n2)cc(OC)c1OC",
      "O=C(Nc1ccc(Oc2ccccc2)cc1)c1cccnc1NCc1ccncc1", "O=C(Nc1ccc(Oc2ccccc2)cc1)c1cccnc1NCc1ccncc1",
      "CNC(=O)c1cc(Oc2ccc3[nH]c(Nc4ccc(Cl)c(C(F)(F)F)c4)nc3c2)ccn1",
      "CNC(=O)c1cc(Oc2ccc3oc(Nc4ccc(Cl)c(OCC5CCC[NH+]5C)c4)nc3c2)ccn1",
      "CNC(=O)c1cc(Oc2ccc3oc(Nc4ccc(Cl)c(OCC5CCC[NH+]5C)c4)nc3c2)ccn1",
      "COc1cc2nccc(Oc3ccc4c(c3)OCCN4C(=O)Nc3ccc(Cl)cc3)c2cc1OC",
      "CNC(=O)c1c(C)oc2cc(Oc3cc[nH+]c4cc(OCCN5CCOCC5)ccc34)ccc12",
      "COc1cc2[nH+]ccc(Oc3ccc4c(C(=O)Nc5ccc(Cl)cc5)cccc4c3)c2cc1OC",
      "COc1cc2[nH+]ccc(Oc3ccc4c(C(=O)Nc5ccc(Cl)cc5)cccc4c3)c2cc1OC",
      "COc1cc2[nH+]ccc(Oc3ccc4c(C(=O)NC5CC5)cccc4c3)c2cc1OC",
      "COc1cc2[nH+]ccc(Oc3ccc4c(C(=O)NC5CC5)cccc4c3)c2cc1OC",
      "Cc1ccc(C(=O)Nc2cc(CCC[NH+](C)C)cc(C(F)(F)F)c2)cc1Nc1ncccc1-c1ccncn1",
      "COc1cc(Nc2nccc(Nc3ccc4c(C)n[nH]c4c3)n2)cc(OC)c1OC",
      "COc1cc(Nc2nccc(N(C)c3ccc4c(C)n[nH]c4c3)n2)cc(OC)c1OC",
      "Cc1ccn(-c2ccc3c(c2)NCC3(C)C)c(=O)c1-c1ccc2nc(N)ncc2c1",
      "Cc1ccn(-c2ccc3c(c2)NCC3(C)C)c(=O)c1-c1ccc2nc(N)ncc2c1",
      "Cc1ccc(C(=O)NCCC2CCCC2)cc1C(=O)Nc1ccc(N)nc1", "Cc1ccc(C(=O)NCCC2CCCC2)cc1C(=O)Nc1ccc(N)nc1",
      "Cc1ccn(-c2cccc(C(F)(F)F)c2)c(=O)c1-c1ccc2nc(N)ncc2c1",
      "Cc1ccn(-c2cccc(C(F)(F)F)c2)c(=O)c1-c1ccc2nc(N)ncc2c1",
      "O=C(Nc1cncnc1)c1c(Cl)ccc2c(Nc3cccc(C(F)(F)F)c3)noc12",
      "O=C(Nc1cncnc1)c1c(Cl)ccc2c(Nc3cccc(C(F)(F)F)c3)noc12",
      "CC1(C)CNc2cc(NC(=O)c3cccnc3NCc3ccncc3)ccc21", "CC1(C)CNc2cc(NC(=O)c3cccnc3NCc3ccncc3)ccc21"
    ]

    for holder in [
        rdSubstructLibrary.CachedSmilesMolHolder(),
        rdSubstructLibrary.CachedTrustedSmilesMolHolder()
    ]:
      for smi in pdb_ligands:
        holder.AddSmiles(smi)

      for patttern in [
          None,
          rdSubstructLibrary.PatternHolder(),
          rdSubstructLibrary.TautomerPatternHolder()
      ]:
        lib = rdSubstructLibrary.SubstructLibrary(holder)
        rdSubstructLibrary.AddPatterns(lib, numThreads=-1)
        self.assertEqual(len(lib.GetMolHolder()), len(lib.GetFpHolder()))
        for smi in pdb_ligands:
          self.assertTrue(lib.CountMatches(Chem.MolFromSmiles(smi)))

  def test_PatternHolder(self):
    for holder in [rdSubstructLibrary.PatternHolder, rdSubstructLibrary.TautomerPatternHolder]:
      fname = os.path.join(os.environ["RDBASE"], "Data", "NCI", "first_5K.smi")
      suppl = Chem.SmilesMolSupplier(fname, delimiter="\t", titleLine=False)
      mols1 = rdSubstructLibrary.CachedTrustedSmilesMolHolder()
      fps1 = holder(2048)
      ssslib1 = rdSubstructLibrary.SubstructLibrary(mols1, fps1)
      mols2 = rdSubstructLibrary.CachedTrustedSmilesMolHolder()
      fps2 = holder()
      ssslib2 = rdSubstructLibrary.SubstructLibrary(mols2, fps2)

      RDLogger.DisableLog('rdApp.error')
      for i in range(0, 1000, 10):
        try:
          mol = suppl[i]
        except Exception:
          continue
        if (not mol):
          continue
        mols1.AddSmiles(Chem.MolToSmiles(mol))
        fps1.AddFingerprint(fps1.MakeFingerprint(mol))
        ssslib2.AddMol(mol)
      RDLogger.EnableLog('rdApp.error')
      query = Chem.MolFromSmarts("N")
      self.assertIsNotNone(query)
      matches1 = sorted(ssslib1.GetMatches(query))
      matches2 = sorted(ssslib2.GetMatches(query))
      self.assertEqual(len(matches1), len(matches2))
      self.assertTrue(all([m1 == matches2[i] for i, m1 in enumerate(matches1)]))

  def testMolBundles(self):
    ssl = rdSubstructLibrary.SubstructLibrary()
    for smi in ('CCOC', 'CCNC', 'COOCOO', 'CCNC', 'CCCC'):
      ssl.AddMol(Chem.MolFromSmiles(smi))
    bndl = Chem.MolBundle()
    for smi in ('COC', 'CCC'):
      bndl.AddMol(Chem.MolFromSmiles(smi))
    self.assertEqual(list(ssl.GetMatches(bndl)), [0, 4])
    bndl.AddMol(Chem.MolFromSmiles('CN'))
    self.assertEqual(list(sorted(ssl.GetMatches(bndl))), [0, 1, 3, 4])

  def testSubstructParameters(self):
    ssl = rdSubstructLibrary.SubstructLibrary()
    for smi in ('C[C@H](F)Cl', 'C[C@@H](F)Cl', 'CC(F)Cl'):
      ssl.AddMol(Chem.MolFromSmiles(smi))
    bndl = Chem.MolBundle()
    for smi in ('C[C@H](F)Cl', ):
      bndl.AddMol(Chem.MolFromSmiles(smi))
    params = Chem.SubstructMatchParameters()
    self.assertEqual(list(sorted(ssl.GetMatches(bndl, params))), [0, 1, 2])

    params.useChirality = True
    self.assertEqual(list(sorted(ssl.GetMatches(bndl, params))), [0])

  def testSearchOrder(self):
    for keyholder in [None, rdSubstructLibrary.KeyFromPropHolder()]:
      ssl = rdSubstructLibrary.SubstructLibrary(rdSubstructLibrary.MolHolder(), keyholder)
      for idx, smi in enumerate(("CCCOC", "CCCCOCC", "CCOC", "COC", "CCCCCOC")):
        m = Chem.MolFromSmiles(smi)
        m.SetProp("_Name", str(idx))
        ssl.AddMol(m)

      ssl.SetSearchOrder((3, 2, 0, 1, 4))
      self.assertEqual(ssl.GetSearchOrder(), (3, 2, 0, 1, 4))
      qm = Chem.MolFromSmiles('COC')
      self.assertEqual(list(ssl.GetMatches(qm, maxResults=2)), [3, 2])
      self.assertEqual(list(ssl.GetMatches(qm, maxResults=2)), [3, 2])
      if keyholder:
        self.assertEqual(keyholder.GetPropName(), "_Name")
        self.assertEqual(list(ssl.GetKeyHolder().GetKeys(ssl.GetMatches(qm, maxResults=2))),
                         ['3', '2'])

      # make sure we can clear the search order:
      ssl.SetSearchOrder(None)
      self.assertEqual(ssl.GetSearchOrder(), ())

      ssl.SetSearchOrder((3, 2, 0, 1, 4))
      self.assertEqual(ssl.GetSearchOrder(), (3, 2, 0, 1, 4))

      ssl.SetSearchOrder([])
      self.assertEqual(ssl.GetSearchOrder(), ())

  def testSearchOrder2(self):
    ssl = rdSubstructLibrary.SubstructLibrary()
    for smi in ("CCCOC", "CCCCOCC", "CCOC", "COC", "CCCCCOC"):
      ssl.AddMol(Chem.MolFromSmiles(smi))

    def setSearchSmallestFirst(sslib):
      searchOrder = list(range(len(sslib)))
      holder = sslib.GetMolHolder()
      searchOrder.sort(key=lambda x, holder=holder: holder.GetMol(x).GetNumAtoms())
      sslib.SetSearchOrder(searchOrder)

    setSearchSmallestFirst(ssl)
    qm = Chem.MolFromSmiles('COC')
    self.assertEqual(list(ssl.GetMatches(qm)), [3, 2, 0, 1, 4])

  def testPropHolder(self):
    for propname in [None, 'foo']:
      if propname is None:
        keyholder = rdSubstructLibrary.KeyFromPropHolder()
      else:
        keyholder = rdSubstructLibrary.KeyFromPropHolder(propname)

      library = rdSubstructLibrary.SubstructLibrary(rdSubstructLibrary.MolHolder(), keyholder)
      m = Chem.MolFromSmiles('CCC')
      if propname is None:
        self.assertEqual(keyholder.GetPropName(), "_Name")
      else:
        self.assertEqual(keyholder.GetPropName(), propname)

      if propname:
        m.SetProp(propname, 'Z11234')
      else:
        m.SetProp("_Name", 'Z11234')

      library.AddMol(m)
      indices = library.GetMatches(m)
      self.assertEqual(['Z11234'], list(library.GetKeyHolder().GetKeys(indices)))

  def test_bad_smiles(self):
    # add mols
    pdb_ligands = [
      "&CCS(=O)(=O)c1ccc(OC)c(Nc2ncc(-c3cccc(-c4ccccn4)c3)o2)c1",
      "&COc1ccc(S(=O)(=O)NCC2CC2)cc1Nc1ncc(-c2cccc(-c3cccnc3)c2)o1",
      "&COc1ccc(-c2oc3ncnc(N)c3c2-c2ccc(NC(=O)Nc3cc(C(F)(F)F)ccc3F)cc2)cc1",
      "&COC(=O)Nc1nc2ccc(Oc3ccc(NC(=O)Nc4cc(C(F)(F)F)ccc4F)cc3)cc2[nH]1",
      "&COc1cc(Nc2ncnc(-c3cccnc3Nc3ccccc3)n2)cc(OC)c1OC",
      "&O=C(Nc1ccc(Oc2ccccc2)cc1)c1cccnc1NCc1ccncc1",
      "&O=C(Nc1ccc(Oc2ccccc2)cc1)c1cccnc1NCc1ccncc1",
      "&CNC(=O)c1cc(Oc2ccc3[nH]c(Nc4ccc(Cl)c(C(F)(F)F)c4)nc3c2)ccn1",
      "&CNC(=O)c1cc(Oc2ccc3oc(Nc4ccc(Cl)c(OCC5CCC[NH+]5C)c4)nc3c2)ccn1",
      "&CNC(=O)c1cc(Oc2ccc3oc(Nc4ccc(Cl)c(OCC5CCC[NH+]5C)c4)nc3c2)ccn1",
      "&COc1cc2nccc(Oc3ccc4c(c3)OCCN4C(=O)Nc3ccc(Cl)cc3)c2cc1OC",
      "&CNC(=O)c1c(C)oc2cc(Oc3cc[nH+]c4cc(OCCN5CCOCC5)ccc34)ccc12",
      "&COc1cc2[nH+]ccc(Oc3ccc4c(C(=O)Nc5ccc(Cl)cc5)cccc4c3)c2cc1OC",
      "&COc1cc2[nH+]ccc(Oc3ccc4c(C(=O)Nc5ccc(Cl)cc5)cccc4c3)c2cc1OC",
      "&COc1cc2[nH+]ccc(Oc3ccc4c(C(=O)NC5CC5)cccc4c3)c2cc1OC",
      "&COc1cc2[nH+]ccc(Oc3ccc4c(C(=O)NC5CC5)cccc4c3)c2cc1OC",
      "&Cc1ccc(C(=O)Nc2cc(CCC[NH+](C)C)cc(C(F)(F)F)c2)cc1Nc1ncccc1-c1ccncn1",
      "&COc1cc(Nc2nccc(Nc3ccc4c(C)n[nH]c4c3)n2)cc(OC)c1OC",
      "&COc1cc(Nc2nccc(N(C)c3ccc4c(C)n[nH]c4c3)n2)cc(OC)c1OC",
      "&Cc1ccn(-c2ccc3c(c2)NCC3(C)C)c(=O)c1-c1ccc2nc(N)ncc2c1",
      "&Cc1ccn(-c2ccc3c(c2)NCC3(C)C)c(=O)c1-c1ccc2nc(N)ncc2c1",
      "&Cc1ccc(C(=O)NCCC2CCCC2)cc1C(=O)Nc1ccc(N)nc1",
      "&Cc1ccc(C(=O)NCCC2CCCC2)cc1C(=O)Nc1ccc(N)nc1",
      "&Cc1ccn(-c2cccc(C(F)(F)F)c2)c(=O)c1-c1ccc2nc(N)ncc2c1",
      "&Cc1ccn(-c2cccc(C(F)(F)F)c2)c(=O)c1-c1ccc2nc(N)ncc2c1",
      "&O=C(Nc1cncnc1)c1c(Cl)ccc2c(Nc3cccc(C(F)(F)F)c3)noc12",
      "&O=C(Nc1cncnc1)c1c(Cl)ccc2c(Nc3cccc(C(F)(F)F)c3)noc12",
      "&CC1(C)CNc2cc(NC(=O)c3cccnc3NCc3ccncc3)ccc21", "&CC1(C)CNc2cc(NC(=O)c3cccnc3NCc3ccncc3)ccc21"
    ]
    # this test is really verbose, so disable the actual output without
    # disabling that logging happens.
    rdBase.LogToPythonLogger()
    pylog = logging.getLogger("rdkit")
    pylog.setLevel(logging.CRITICAL)
    for holder in [
        rdSubstructLibrary.CachedSmilesMolHolder(),
        rdSubstructLibrary.CachedTrustedSmilesMolHolder()
    ]:
      for smi in pdb_ligands:
        holder.AddSmiles(smi)
      lib = rdSubstructLibrary.SubstructLibrary(holder)
      # this should excercise the logger
      smi = "CCS(=O)(=O)c1ccc(OC)c(Nc2ncc(-c3cccc(-c4ccccn4)c3)o2)c1"
      self.assertEqual(0, lib.CountMatches(Chem.MolFromSmiles(smi)))

      # test add patterns
      rdSubstructLibrary.AddPatterns(lib, -1)
    pylog.setLevel(logging.WARN)
    rdBase.LogToCppStreams()

  @unittest.skipIf(not rdBase._serializationEnabled, "not built with serialization support")
  def test_using_xqms(self):
    smis = ["COCC=O", "COOCC=O", "COOOCC=O", "COOOOCC=O"]

    for holder in [
        rdSubstructLibrary.CachedSmilesMolHolder(),
        rdSubstructLibrary.CachedTrustedSmilesMolHolder()
    ]:
      for smi in smis:
        holder.AddSmiles(smi)
      fph = rdSubstructLibrary.TautomerPatternHolder()
      lib = rdSubstructLibrary.SubstructLibrary(holder)

      mol = Chem.MolFromSmiles("COCC")
      xqm = rdGeneralizedSubstruct.CreateExtendedQueryMol(mol)
      res = lib.GetMatches(xqm)
      self.assertEqual(list(res), [0])
      self.assertTrue(lib.HasMatch(xqm))
      self.assertEqual(lib.CountMatches(xqm), 1)

      mol = Chem.MolFromSmiles("COC=CO")
      xqm = rdGeneralizedSubstruct.CreateExtendedQueryMol(mol)
      res = lib.GetMatches(xqm)
      self.assertEqual(list(res), [0])
      self.assertTrue(lib.HasMatch(xqm))
      self.assertEqual(lib.CountMatches(xqm), 1)

      mol = Chem.MolFromSmiles("COCC |LN:1:1.3|")
      xqm = rdGeneralizedSubstruct.CreateExtendedQueryMol(mol)
      res = lib.GetMatches(xqm)
      self.assertEqual(list(res), [0, 1, 2])
      self.assertTrue(lib.HasMatch(xqm))
      self.assertEqual(lib.CountMatches(xqm), 3)

      mol = Chem.MolFromSmiles("COC=CO |LN:1:1.3|")
      xqm = rdGeneralizedSubstruct.CreateExtendedQueryMol(mol)
      res = lib.GetMatches(xqm)
      self.assertEqual(list(res), [0, 1, 2])
      self.assertTrue(lib.HasMatch(xqm))
      self.assertEqual(lib.CountMatches(xqm), 3)

      mol = Chem.MolFromSmiles("CNC=CO |LN:1:1.3|")
      xqm = rdGeneralizedSubstruct.CreateExtendedQueryMol(mol)
      res = lib.GetMatches(xqm)
      self.assertEqual(list(res), [])
      self.assertFalse(lib.HasMatch(xqm))
      self.assertEqual(lib.CountMatches(xqm), 0)


if __name__ == '__main__':
  unittest.main()



#
#  Copyright (C) 2003-2024 Greg Landrum and other RDKit contributors
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
"""basic unit testing code for atoms

"""
import unittest

from rdkit import Chem


class TestCase(unittest.TestCase):

  def setUp(self):
    #print '\n%s: '%self.shortDescription(),
    self.m = Chem.MolFromSmiles('CC(=O)CCSC')

  def test1Implicit(self):
    " testing ImplicitValence "
    a = self.m.GetAtoms()[0]
    iV = a.GetValence(which=Chem.ValenceType.IMPLICIT)
    assert iV == 3
    assert self.m.GetAtomWithIdx(1).GetValence(which=Chem.ValenceType.IMPLICIT) == 0
    assert self.m.GetAtomWithIdx(2).GetValence(which=Chem.ValenceType.IMPLICIT) == 0
    assert self.m.GetAtomWithIdx(3).GetValence(which=Chem.ValenceType.IMPLICIT) == 2

  def test2BondIter(self):
    " testing bond iteration "
    a = self.m.GetAtomWithIdx(1)
    bs = a.GetBonds()
    r = []
    for b in bs:
      r.append(b)
    assert len(r) == 3

  def test3GetBond(self):
    " testing GetBondBetweenAtoms(idx,idx) "
    b = self.m.GetBondBetweenAtoms(1, 2)
    assert b.GetBondType() == Chem.BondType.DOUBLE, 'GetBond failed'

  def test4Props(self):
    " testing atomic props "
    a = self.m.GetAtomWithIdx(1)
    assert a.GetSymbol() == 'C'
    assert a.GetAtomicNum() == 6
    assert a.GetFormalCharge() == 0
    assert a.GetDegree() == 3
    assert a.GetValence(which=Chem.ValenceType.IMPLICIT) == 0
    assert a.GetValence(which=Chem.ValenceType.EXPLICIT) == 4

  def test5Setters(self):
    " testing setting atomic props "
    a = Chem.Atom(6)
    assert a.GetSymbol() == 'C'
    assert a.GetAtomicNum() == 6
    a.SetFormalCharge(1)
    assert a.GetFormalCharge() == 1
    assert a.GetValence(which=Chem.ValenceType.IMPLICIT) == 0


if __name__ == '__main__':
  unittest.main()



"""
## Automatically adapted for numpy.oldnumeric Jun 27, 2008 by -c
Replaced numpy.oldnumeric with numpy methods - Jan 2015, PGedeck
"""
#pylint: disable=E1101,C0111,R0904

import copy
import math
import unittest

import numpy as np

import rdkit.Numerics.rdAlignment as rdAlg
from rdkit import Geometry


def lstFeq(l1, l2, tol=1.e-4):
  if len(list(l1)) != len(list(l2)):
    return 0
  for i in range(len(list(l1))):
    if not feq(l1[i], l2[i], tol):
      return 0
  return 1


def feq(v1, v2, tol2=1e-4):
  return abs(v1 - v2) <= tol2


def transformPoint(trans, pt):
  pt2 = copy.copy(list(pt))
  pt2.append(1.0)
  pt2 = np.array(pt2)
  res = np.dot(trans, pt2)
  return res[:3]


class TestCase(unittest.TestCase):

  def test1Basic(self):
    # passing two numeric arrays
    refPts = np.zeros((2, 3), float)
    prbPts = np.zeros((2, 3), float)

    refPts[1, 0] = 1.0

    prbPts[0, 0] = 2.0
    prbPts[0, 1] = 2.0
    prbPts[1, 0] = 2.0
    prbPts[1, 1] = 3.0

    res = rdAlg.GetAlignmentTransform(refPts, prbPts)
    self.assertTrue(feq(res[0], 0.0))
    refLst = list(refPts)
    cnt = 0
    for item in list(prbPts):
      self.assertTrue(lstFeq(transformPoint(res[1], item), refLst[cnt]))
      cnt += 1

    # repeat with with lists or tuples
    refPts = ((0.0, 0.0, 0.0), (1.0, 0.0, 0.0))
    prbPts = ((2.0, 2.0, 0.0), (2.0, 3.0, 0.0))
    res = rdAlg.GetAlignmentTransform(refPts, prbPts)
    self.assertTrue(feq(res[0], 0.0))

    refPts = [[0.0, 0.0, 0.0], [1.0, 0.0, 0.0]]
    prbPts = [[2.0, 2.0, 0.0], [2.0, 3.0, 0.0]]
    res = rdAlg.GetAlignmentTransform(refPts, prbPts)
    self.assertTrue(feq(res[0], 0.0))

    # mix it up
    refPts = np.zeros((2, 3), float)
    refPts[1, 0] = 1.0
    res = rdAlg.GetAlignmentTransform(refPts, prbPts)
    self.assertTrue(feq(res[0], 0.0))

  def test2Weights(self):
    refPts = np.array([[-math.cos(math.pi / 6), -math.sin(math.pi / 6), 0.0],
                       [math.cos(math.pi / 6), -math.sin(math.pi / 6), 0.0], [0.0, 1.0, 0.0]],
                      float)
    prbPts = np.array(
      [[-2 * math.sin(math.pi / 6) + 3.0, 2 * math.cos(math.pi / 6), 4.0],
       [-2 * math.sin(math.pi / 6) + 3.0, -2 * math.cos(math.pi / 6), 4.0], [5.0, 0.0, 4.0]], float)
    res = rdAlg.GetAlignmentTransform(refPts, prbPts)
    self.assertTrue(feq(res[0], 3.0))
    target = [[-1.732, -1., 0.], [1.732, -1., 0.], [0., 2., 0.]]
    cnt = 0
    for item in list(prbPts):
      self.assertTrue(lstFeq(transformPoint(res[1], item), target[cnt]))
      cnt += 1

    weights = np.array([1.0, 1.0, 2.0], float)
    res = rdAlg.GetAlignmentTransform(refPts, prbPts, weights)
    self.assertTrue(feq(res[0], 3.75))
    cnt = 0
    target = [[-1.732, -1.25, 0.], [1.732, -1.25, 0.], [0., 1.75, 0.]]
    for item in list(prbPts):
      self.assertTrue(lstFeq(transformPoint(res[1], item), target[cnt]))
      cnt += 1
    weights = [1.0, 1.0, 2.0]
    res = rdAlg.GetAlignmentTransform(refPts, prbPts, weights)
    self.assertTrue(feq(res[0], 3.75))

    weights = [1.0, 2.0, 2.0]
    res = rdAlg.GetAlignmentTransform(refPts, prbPts, weights)
    self.assertTrue(feq(res[0], 4.8))

  def test3tetra(self):
    refPts = np.array([[0.0, 0.0, 0.0], [1.0, 0.0, 0.0], [0.0, 1.0, 0.0], [0.0, 0.0, 1.0]], float)
    prbPts = np.array([[2.0, 2.0, 3.0], [3.0, 2.0, 3.0], [2.0, 3.0, 3.0]], float)
    self.assertRaises(ValueError, lambda: rdAlg.GetAlignmentTransform(refPts, prbPts))

    prbPts = np.array([[2.0, 2.0, 3.0], [3.0, 2.0, 3.0], [2.0, 3.0, 3.0], [2.0, 2.0, 4.0]], float)
    res = rdAlg.GetAlignmentTransform(refPts, prbPts)
    self.assertTrue(feq(res[0], 0.0))

    wts = [1.0, 1.0, 1.0]
    self.assertRaises(ValueError, lambda: rdAlg.GetAlignmentTransform(refPts, prbPts, wts))

    wts = [1.0, 1.0, 1.0, 1.0]
    res = rdAlg.GetAlignmentTransform(refPts, prbPts, wts)
    self.assertTrue(feq(res[0], 0.0))

    # test reflection
    prbPts = np.array([[2.0, 2.0, 3.0], [3.0, 2.0, 3.0], [2.0, 2.0, 4.0], [2.0, 3.0, 3.0]], float)
    res = rdAlg.GetAlignmentTransform(refPts, prbPts, wts)
    self.assertTrue(feq(res[0], 1.0))

    res = rdAlg.GetAlignmentTransform(refPts, prbPts, wts, 1)
    self.assertTrue(feq(res[0], 0.0))
    cnt = 0
    refLst = list(refPts)
    for item in list(prbPts):
      self.assertTrue(lstFeq(transformPoint(res[1], item), refLst[cnt]))
      cnt += 1

  def test4points(self):
    refPts = (
      Geometry.Point3D(0.0, 0.0, 0.0),
      Geometry.Point3D(1.0, 0.0, 0.0),
      Geometry.Point3D(0.0, 1.0, 0.0),
      Geometry.Point3D(0.0, 0.0, 1.0),
    )
    prbPts = (
      Geometry.Point3D(2.0, 2.0, 3.0),
      Geometry.Point3D(3.0, 2.0, 3.0),
      Geometry.Point3D(2.0, 3.0, 3.0),
      Geometry.Point3D(2.0, 2.0, 4.0),
    )

    res = rdAlg.GetAlignmentTransform(refPts, prbPts)
    self.assertTrue(feq(res[0], 0.0))

  def test5errorHandling(self):
    refPts = (
      Geometry.Point3D(0.0, 0.0, 0.0),
      Geometry.Point3D(1.0, 0.0, 0.0),
      Geometry.Point3D(0.0, 1.0, 0.0),
      Geometry.Point3D(0.0, 0.0, 1.0),
    )
    prbPts = (
      1,
      2,
      3,
      4,
    )
    self.assertRaises(ValueError, lambda: rdAlg.GetAlignmentTransform(refPts, prbPts))
    prbPts = ()
    self.assertRaises(ValueError, lambda: rdAlg.GetAlignmentTransform(refPts, prbPts))

    prbPts = 1
    self.assertRaises(ValueError, lambda: rdAlg.GetAlignmentTransform(refPts, prbPts))

    prbPts = (
      Geometry.Point3D(2.0, 2.0, 3.0),
      Geometry.Point3D(3.0, 2.0, 3.0),
      Geometry.Point3D(2.0, 3.0, 3.0),
      (2.0, 2.0, 5.0),
    )
    self.assertRaises(ValueError, lambda: rdAlg.GetAlignmentTransform(refPts, prbPts))


if __name__ == '__main__':
  print("Testing Alignment Wrapper code:")
  unittest.main()



#
#  Copyright (C) 2003-2006  greg Landrum and Rational Discovery LLC
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
"""unit testing code for the lazy signature generator

"""
import unittest

from rdkit import Chem
from rdkit.Chem.Pharm2D import SigFactory

try:
  from rdkit.Chem.Pharm2D import LazyGenerator
except NotImplementedError:
  LazyGenerator = None


class TestCase(unittest.TestCase):  # pragma: nocover

  def getFactory(self):
    factory = SigFactory.SigFactory()
    factory.SetPatternsFromSmarts(['O', 'N'])
    factory.SetBins([(0, 2), (2, 5), (5, 8)])
    factory.SetMinCount(2)
    factory.SetMaxCount(3)
    return factory

  def test_NotImplemented(self):
    self.assertIsNone(LazyGenerator, 'Review LazyGenerator unit tests')

  @unittest.skipIf(LazyGenerator is None, 'LazyGenerator implementation incomplete')
  def test1_simple(self):
    mol = Chem.MolFromSmiles('OCC(=O)CCCN')
    factory = self.getFactory()
    sig = factory.GetSignature()
    assert sig.GetSize() == 105, f'bad signature size: {sig.GetSize()}'
    sig.SetIncludeBondOrder(0)
    gen = LazyGenerator.Generator(sig, mol)
    assert len(gen) == sig.GetSize(), f'length mismatch {len(gen)}!={sig.GetSize()}'

    tgt = (1, 5, 48)
    for bit in tgt:
      assert gen[bit], f'bit {bit} not properly set'
      assert gen.GetBit(bit), f'bit {bit} not properly set'
      assert not gen[bit + 50], f'bit {bit + 100} improperly set'

    sig = factory.GetSignature()
    assert sig.GetSize() == 105, f'bad signature size: {sig.GetSize()}'
    sig.SetIncludeBondOrder(1)
    gen = LazyGenerator.Generator(sig, mol)
    assert len(gen) == sig.GetSize(), f'length mismatch {len(gen)}!={sig.GetSize()}'

    tgt = (1, 4, 5, 45)
    for bit in tgt:
      assert gen[bit], f'bit {bit} not properly set'
      assert gen.GetBit(bit), f'bit {bit} not properly set'
      assert not gen[bit + 50], f'bit {bit + 100} improperly set'

    try:
      gen[sig.GetSize() + 1]
    except IndexError:
      ok = 1
    else:
      ok = 0
    assert ok, 'accessing bogus bit did not fail'
    try:
      gen[-1]
    except IndexError:
      ok = 1
    else:
      ok = 0
    assert ok, 'accessing bogus bit did not fail'


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



from __future__ import print_function

import os
import re
from datetime import datetime

from setuptools import setup

pkg_version = ''
src_dir = os.path.realpath(__file__)
have_src_dir = os.path.isfile(src_dir)
i = 0
while (i < 2 and have_src_dir):
  i += 1
  src_dir = os.path.dirname(src_dir)
  have_src_dir = os.path.isdir(src_dir)
if (not have_src_dir):
  raise OSError('Could not find SRC_DIR, got: ' + str(src_dir))
# parse root CMakeLists.txt and Code/cmake/Modules/RDKitUtils.cmake
root_cmakelists_path = os.path.join(src_dir, 'CMakeLists.txt')
rdkitutils_path = os.path.join(src_dir, 'Code', 'cmake', 'Modules', 'RDKitUtils.cmake')

var_dict = {}
for file in (root_cmakelists_path, rdkitutils_path):
  with open(file, 'rt') as hnd:
    # vars we want to read
    var_set = set(['RDKit_Year', 'RDKit_Month', 'RDKit_Revision', 'RDKit_ABI', 'RDKit_RELEASENAME'])
    line = hnd.readline()
    while (line):
      # is this an uncommented set command?
      m = re.match(r'^\s*set\s*\((\w+)\s*\"(.*)\"\s*\)', line, re.IGNORECASE)
      # if it is
      if (m is not None):
        # extract the var name
        var_name = m.group(1)
        if (var_name in var_set):
          # if the var name is in the vars we want to read
          var_value = m.group(2)
          keepLooping = True
          while (keepLooping):
            # recursively replace variables we already found
            m = re.match(r'^.*\${(\w+)}', var_value)
            keepLooping = (m is not None)
            if (keepLooping):
              v = var_dict.get(m.group(1))
              # if the variable is not defined, remove
              # the preceding dot
              eat_dot = 0
              if (v is None):
                v = ''
                eat_dot = 1
              # replace variable name with its value
              s = m.start(1) - (2 + eat_dot)
              e = m.end(1) + 1
              var_value = var_value[:s] + v + var_value[e:]
          # assign variable value and keep parsing
          var_dict[var_name] = var_value
      line = hnd.readline()
d = datetime.today().strftime('%Y%m%d')
rdkitVersion = var_dict.get('RDKit_RELEASENAME')
if (rdkitVersion is not None):
  if rdkitVersion.endswith('.dev1'):
    pkg_version = rdkitVersion[:-1] + d
  else:
    pkg_version = rdkitVersion
else:
  # if extracting rdkitVersion somehow failed, use the date
  pkg_version = d

print('rdkitVersion:', pkg_version)
setup(rdkitVersion=pkg_version)



import csv
import io
import logging
import os

import freewilson as fw

from rdkit import Chem, rdBase

PATH = os.path.join(os.path.dirname(fw.__file__), 'data')
assert os.path.exists(PATH), PATH


def test_chembl():
  logging.getLogger().setLevel(logging.INFO)
  smilesfile = os.path.join(PATH, "CHEMBL2321810.smi")
  scaffoldfile = os.path.join(PATH, "CHEMBL2321810_scaffold.mol")
  csvfile = os.path.join(PATH, "CHEMBL2321810_act.csv")
  assert os.path.exists(smilesfile)
  mols = []
  for line in open(smilesfile):
    smiles, name = line.strip().split()
    m = Chem.MolFromSmiles(smiles)
    m.SetProp("_Name", name)
    mols.append(m)

  scaffold = Chem.MolFromMolBlock(open(scaffoldfile).read())
  data = {k: float(v) for k, v in list(csv.reader(open(csvfile)))[1:]}

  scores = [data[m.GetProp("_Name")] for m in mols]
  assert mols and len(mols) == len(scores)

  with rdBase.BlockLogs():
    free = fw.FWDecompose(scaffold, mols, scores)
  # let's make sure the r squared is decent
  assert free.r2 > 0.8

  # assert we get something
  preds = list(fw.FWBuild(free))
  assert len(preds)

  # check to see that the prediction filters work
  preds2 = list(fw.FWBuild(free, pred_filter=lambda x: x > 8))
  assert len(preds2)
  assert len([p for p in preds if p.prediction > 8]) == len(list(preds2))

  # check to see that the R groups are output in order, i.e. R10 after R3
  s = io.StringIO()
  fw.predictions_to_csv(s, free, preds2)
  assert s.getvalue()

  s2 = io.StringIO(s.getvalue())
  for i, row in enumerate(csv.reader(s2)):
    if i == 0:
      assert row == ['smiles', 'prediction', 'Core_smiles', 'R1_smiles', 'R3_smiles', 'R10_smiles']
  assert i > 0


def test_multicore():
  # test that we can add rgroups for later cores and not throw an exception
  scaffolds = [Chem.MolFromSmiles("c1ccccc1[*].NC=O"), Chem.MolFromSmiles("C1CCCCC1")]
  mols = [
    Chem.MolFromSmiles(x) for x in [
      'c1ccccc1CC2CNC2C(=O)N', 'Cc1ccccc1CC2CNC2C(=O)N', 'Cc1ccccc1CC2CNCC(=O)NC2',
      'C3c1ccccc1CC2CNC2C(=O)N3', 'C1CCCCC1F', 'ClC1CCCCC1F'
    ]
  ]
  decomp = fw.FWDecompose(scaffolds, mols, [1, 2, 3, 4, 5, 6])
  s = io.StringIO()
  fw.predictions_to_csv(s, decomp, fw.FWBuild(decomp))



# must be run from this directory

import csv
import os
import sys

py3 = sys.version_info[0] == 3

pains_a = [
  'ene_six_het_A(483)', 'hzone_phenol_A(479)', 'anil_di_alk_A(478)', 'indol_3yl_alk(461)',
  'quinone_A(370)', 'azo_A(324)', 'imine_one_A(321)', 'mannich_A(296)', 'anil_di_alk_B(251)',
  'anil_di_alk_C(246)', 'ene_rhod_A(235)', 'hzone_phenol_B(215)', 'ene_five_het_A(201)',
  'anil_di_alk_D(198)', 'imine_one_isatin(189)', 'anil_di_alk_E(186)'
]

pains_b = [
  'thiaz_ene_A(128)', 'pyrrole_A(118)', 'catechol_A(92)', 'ene_five_het_B(90)',
  'imine_one_fives(89)', 'ene_five_het_C(85)', 'hzone_pipzn(79)', 'keto_keto_beta_A(68)',
  'hzone_pyrrol(64)', 'ene_one_ene_A(57)', 'cyano_ene_amine_A(56)', 'ene_five_one_A(55)',
  'cyano_pyridone_A(54)', 'anil_alk_ene(51)', 'amino_acridine_A(46)', 'ene_five_het_D(46)',
  'thiophene_amino_Aa(45)', 'ene_five_het_E(44)', 'sulfonamide_A(43)', 'thio_ketone(43)',
  'sulfonamide_B(41)', 'anil_no_alk(40)', 'thiophene_amino_Ab(40)', 'het_pyridiniums_A(39)',
  'anthranil_one_A(38)', 'cyano_imine_A(37)', 'diazox_sulfon_A(36)', 'hzone_anil_di_alk(35)',
  'rhod_sat_A(33)', 'hzone_enamin(30)', 'pyrrole_B(29)', 'thiophene_hydroxy(28)',
  'cyano_pyridone_B(27)', 'imine_one_sixes(27)', 'dyes5A(27)', 'naphth_amino_A(25)',
  'naphth_amino_B(25)', 'ene_one_ester(24)', 'thio_dibenzo(23)', 'cyano_cyano_A(23)',
  'hzone_acyl_naphthol(22)', 'het_65_A(21)', 'imidazole_A(19)', 'ene_cyano_A(19)',
  'anthranil_acid_A(19)', 'dyes3A(19)', 'dhp_bis_amino_CN(19)', 'het_6_tetrazine(18)',
  'ene_one_hal(17)', 'cyano_imine_B(17)', 'thiaz_ene_B(17)', 'ene_rhod_B(16)',
  'thio_carbonate_A(15)', 'anil_di_alk_furan_A(15)', 'ene_five_het_F(15)'
]

pains_c = [
  'anil_di_alk_F(14)', 'hzone_anil(14)', 'het_5_pyrazole_OH(14)', 'het_thio_666_A(13)',
  'styrene_A(13)', 'ene_rhod_C(13)', 'dhp_amino_CN_A(13)', 'cyano_imine_C(12)', 'thio_urea_A(12)',
  'thiophene_amino_B(12)', 'keto_keto_beta_B(12)', 'keto_phenone_A(11)', 'cyano_pyridone_C(11)',
  'thiaz_ene_C(11)', 'hzone_thiophene_A(11)', 'ene_quin_methide(10)', 'het_thio_676_A(10)',
  'ene_five_het_G(10)', 'acyl_het_A(9)', 'anil_di_alk_G(9)', 'dhp_keto_A(9)', 'thio_urea_B(9)',
  'anil_alk_bim(9)', 'imine_imine_A(9)', 'thio_urea_C(9)', 'imine_one_fives_B(9)',
  'dhp_amino_CN_B(9)', 'anil_OC_no_alk_A(8)', 'het_thio_66_one(8)', 'styrene_B(8)',
  'het_thio_5_A(8)', 'anil_di_alk_ene_A(8)', 'ene_rhod_D(8)', 'ene_rhod_E(8)', 'anil_OH_alk_A(8)',
  'pyrrole_C(8)', 'thio_urea_D(8)', 'thiaz_ene_D(8)', 'ene_rhod_F(8)', 'thiaz_ene_E(8)',
  'het_65_B(7)', 'keto_keto_beta_C(7)', 'het_66_A(7)', 'thio_urea_E(7)', 'thiophene_amino_C(7)',
  'hzone_phenone(7)', 'ene_rhod_G(7)', 'ene_cyano_B(7)', 'dhp_amino_CN_C(7)', 'het_5_A(7)',
  'ene_five_het_H(6)', 'thio_amide_A(6)', 'ene_cyano_C(6)', 'hzone_furan_A(6)', 'anil_di_alk_H(6)',
  'het_65_C(6)', 'thio_urea_F(6)', 'ene_five_het_I(6)', 'keto_keto_gamma(5)', 'quinone_B(5)',
  'het_6_pyridone_OH(5)', 'hzone_naphth_A(5)', 'thio_ester_A(5)', 'ene_misc_A(5)',
  'cyano_pyridone_D(5)', 'het_65_Db(5)', 'het_666_A(5)', 'diazox_sulfon_B(5)', 'anil_NH_alk_A(5)',
  'sulfonamide_C(5)', 'het_thio_N_55(5)', 'keto_keto_beta_D(5)', 'ene_rhod_H(5)', 'imine_ene_A(5)',
  'het_thio_656a(5)', 'pyrrole_D(5)', 'pyrrole_E(5)', 'thio_urea_G(5)', 'anisol_A(5)',
  'pyrrole_F(5)', 'dhp_amino_CN_D(5)', 'thiazole_amine_A(4)', 'het_6_imidate_A(4)',
  'anil_OC_no_alk_B(4)', 'styrene_C(4)', 'azulene(4)', 'furan_acid_A(4)', 'cyano_pyridone_E(4)',
  'anil_alk_thio(4)', 'anil_di_alk_I(4)', 'het_thio_6_furan(4)', 'anil_di_alk_ene_B(4)',
  'imine_one_B(4)', 'anil_OC_alk_A(4)', 'ene_five_het_J(4)', 'pyrrole_G(4)', 'ene_five_het_K(4)',
  'cyano_ene_amine_B(4)', 'thio_ester_B(4)', 'ene_five_het_L(4)', 'hzone_thiophene_B(4)',
  'dhp_amino_CN_E(4)', 'het_5_B(4)', 'imine_imine_B(3)', 'thiazole_amine_B(3)',
  'imine_ene_one_A(3)', 'diazox_A(3)', 'ene_one_A(3)', 'anil_OC_no_alk_C(3)', 'thiazol_SC_A(3)',
  'het_666_B(3)', 'furan_A(3)', 'colchicine_A(3)', 'thiophene_C(3)', 'anil_OC_alk_B(3)',
  'het_thio_66_A(3)', 'rhod_sat_B(3)', 'ene_rhod_I(3)', 'keto_thiophene(3)', 'imine_imine_C(3)',
  'het_65_pyridone_A(3)', 'thiazole_amine_C(3)', 'het_thio_pyr_A(3)', 'melamine_A(3)',
  'anil_NH_alk_B(3)', 'rhod_sat_C(3)', 'thiophene_amino_D(3)', 'anil_OC_alk_C(3)',
  'het_thio_65_A(3)', 'het_thio_656b(3)', 'thiazole_amine_D(3)', 'thio_urea_H(3)',
  'cyano_pyridone_F(3)', 'rhod_sat_D(3)', 'ene_rhod_J(3)', 'imine_phenol_A(3)',
  'thio_carbonate_B(3)', 'het_thio_N_5A(3)', 'het_thio_N_65A(3)', 'anil_di_alk_J(3)',
  'pyrrole_H(3)', 'ene_cyano_D(3)', 'cyano_cyano_B(3)', 'ene_five_het_M(3)', 'cyano_ene_amine_C(3)',
  'thio_urea_I(3)', 'dhp_amino_CN_F(3)', 'anthranil_acid_B(3)', 'diazox_B(3)', 'thio_aldehyd_A(3)',
  'thio_amide_B(2)', 'imidazole_B(2)', 'thiazole_amine_E(2)', 'thiazole_amine_F(2)',
  'thio_ester_C(2)', 'ene_one_B(2)', 'quinone_C(2)', 'keto_naphthol_A(2)', 'thio_amide_C(2)',
  'phthalimide_misc(2)', 'sulfonamide_D(2)', 'anil_NH_alk_C(2)', 'het_65_E(2)', 'hzide_naphth(2)',
  'anisol_B(2)', 'thio_carbam_ene(2)', 'thio_amide_D(2)', 'het_65_Da(2)', 'thiophene_D(2)',
  'het_thio_6_ene(2)', 'cyano_keto_A(2)', 'anthranil_acid_C(2)', 'naphth_amino_C(2)',
  'naphth_amino_D(2)', 'thiazole_amine_G(2)', 'het_66_B(2)', 'coumarin_A(2)', 'anthranil_acid_D(2)',
  'het_66_C(2)', 'thiophene_amino_E(2)', 'het_6666_A(2)', 'sulfonamide_E(2)', 'anil_di_alk_K(2)',
  'het_5_C(2)', 'ene_six_het_B(2)', 'steroid_A(2)', 'het_565_A(2)', 'thio_imine_ium(2)',
  'anthranil_acid_E(2)', 'hzone_furan_B(2)', 'thiophene_E(2)', 'ene_misc_B(2)', 'het_thio_5_B(2)',
  'thiophene_amino_F(2)', 'anil_OC_alk_D(2)', 'tert_butyl_A(2)', 'thio_urea_J(2)',
  'het_thio_65_B(2)', 'coumarin_B(2)', 'thio_urea_K(2)', 'thiophene_amino_G(2)', 'anil_NH_alk_D(2)',
  'het_thio_5_C(2)', 'thio_keto_het(2)', 'het_thio_N_5B(2)', 'quinone_D(2)',
  'anil_di_alk_furan_B(2)', 'ene_six_het_C(2)', 'het_55_A(2)', 'het_thio_65_C(2)', 'hydroquin_A(2)',
  'anthranil_acid_F(2)', 'pyrrole_I(2)', 'thiophene_amino_H(2)', 'imine_one_fives_C(2)',
  'keto_phenone_zone_A(2)', 'dyes7A(2)', 'het_pyridiniums_B(2)', 'het_5_D(2)',
  'thiazole_amine_H(1)', 'thiazole_amine_I(1)', 'het_thio_N_5C(1)', 'sulfonamide_F(1)',
  'thiazole_amine_J(1)', 'het_65_F(1)', 'keto_keto_beta_E(1)', 'ene_five_one_B(1)',
  'keto_keto_beta_zone(1)', 'thio_urea_L(1)', 'het_thio_urea_ene(1)', 'cyano_amino_het_A(1)',
  'tetrazole_hzide(1)', 'imine_naphthol_A(1)', 'misc_anisole_A(1)', 'het_thio_665(1)',
  'anil_di_alk_L(1)', 'colchicine_B(1)', 'misc_aminoacid_A(1)', 'imidazole_amino_A(1)',
  'phenol_sulfite_A(1)', 'het_66_D(1)', 'misc_anisole_B(1)', 'tetrazole_A(1)', 'het_65_G(1)',
  'misc_trityl_A(1)', 'misc_pyridine_OC(1)', 'het_6_hydropyridone(1)', 'misc_stilbene(1)',
  'misc_imidazole(1)', 'anil_NH_no_alk_A(1)', 'het_6_imidate_B(1)', 'anil_alk_B(1)',
  'styrene_anil_A(1)', 'misc_aminal_acid(1)', 'anil_no_alk_D(1)', 'anil_alk_C(1)',
  'misc_anisole_C(1)', 'het_465_misc(1)', 'anthranil_acid_G(1)', 'anil_di_alk_M(1)',
  'anthranil_acid_H(1)', 'thio_urea_M(1)', 'thiazole_amine_K(1)', 'het_thio_5_imine_A(1)',
  'thio_amide_E(1)', 'het_thio_676_B(1)', 'sulfonamide_G(1)', 'thio_thiomorph_Z(1)',
  'naphth_ene_one_A(1)', 'naphth_ene_one_B(1)', 'amino_acridine_A(1)', 'keto_phenone_B(1)',
  'hzone_acid_A(1)', 'sulfonamide_H(1)', 'het_565_indole(1)', 'pyrrole_J(1)', 'pyrazole_amino_B(1)',
  'pyrrole_K(1)', 'anthranil_acid_I(1)', 'thio_amide_F(1)', 'ene_one_C(1)', 'het_65_H(1)',
  'cyano_imine_D(1)', 'cyano_misc_A(1)', 'ene_misc_C(1)', 'het_66_E(1)', 'keto_keto_beta_F(1)',
  'misc_naphthimidazole(1)', 'naphth_ene_one_C(1)', 'keto_phenone_C(1)', 'coumarin_C(1)',
  'thio_est_cyano_A(1)', 'het_65_imidazole(1)', 'anthranil_acid_J(1)', 'colchicine_het(1)',
  'ene_misc_D(1)', 'indole_3yl_alk_B(1)', 'anil_OH_no_alk_A(1)', 'thiazole_amine_L(1)',
  'pyrazole_amino_A(1)', 'het_thio_N_5D(1)', 'anil_alk_indane(1)', 'anil_di_alk_N(1)',
  'het_666_C(1)', 'ene_one_D(1)', 'anil_di_alk_indol(1)', 'anil_no_alk_indol_A(1)',
  'dhp_amino_CN_G(1)', 'anil_di_alk_dhp(1)', 'anthranil_amide_A(1)', 'hzone_anthran_Z(1)',
  'ene_one_amide_A(1)', 'het_76_A(1)', 'thio_urea_N(1)', 'anil_di_alk_coum(1)',
  'ene_one_amide_B(1)', 'het_thio_656c(1)', 'het_5_ene(1)', 'thio_imide_A(1)', 'dhp_amidine_A(1)',
  'thio_urea_O(1)', 'anil_di_alk_O(1)', 'thio_urea_P(1)', 'het_pyraz_misc(1)', 'diazox_C(1)',
  'diazox_D(1)', 'misc_cyclopropane(1)', 'imine_ene_one_B(1)', 'coumarin_D(1)', 'misc_furan_A(1)',
  'rhod_sat_E(1)', 'rhod_sat_imine_A(1)', 'rhod_sat_F(1)', 'het_thio_5_imine_B(1)',
  'het_thio_5_imine_C(1)', 'ene_five_het_N(1)', 'thio_carbam_A(1)', 'misc_anilide_A(1)',
  'misc_anilide_B(1)', 'mannich_B(1)', 'mannich_catechol_A(1)', 'anil_alk_D(1)', 'het_65_I(1)',
  'misc_urea_A(1)', 'imidazole_C(1)', 'styrene_imidazole_A(1)', 'thiazole_amine_M(1)',
  'misc_pyrrole_thiaz(1)', 'pyrrole_L(1)', 'het_thio_65_D(1)', 'ene_misc_E(1)', 'thio_cyano_A(1)',
  'cyano_amino_het_B(1)', 'cyano_pyridone_G(1)', 'het_65_J(1)', 'ene_one_yne_A(1)',
  'anil_OH_no_alk_B(1)', 'hzone_acyl_misc_A(1)', 'thiophene_F(1)', 'anil_OC_alk_E(1)',
  'anil_OC_alk_F(1)', 'het_65_K(1)', 'het_65_L(1)', 'coumarin_E(1)', 'coumarin_F(1)',
  'coumarin_G(1)', 'coumarin_H(1)', 'het_thio_67_A(1)', 'sulfonamide_I(1)', 'het_65_mannich(1)',
  'anil_alk_A(1)', 'het_5_inium(1)', 'anil_di_alk_P(1)', 'thio_urea_Q(1)', 'thio_pyridine_A(1)',
  'melamine_B(1)', 'misc_phthal_thio_N(1)', 'hzone_acyl_misc_B(1)', 'tert_butyl_B(1)',
  'diazox_E(1)', 'anil_NH_no_alk_B(1)', 'anil_no_alk_A(1)', 'anil_no_alk_B(1)',
  'thio_ene_amine_A(1)', 'het_55_B(1)', 'cyanamide_A(1)', 'ene_one_one_A(1)', 'ene_six_het_D(1)',
  'ene_cyano_E(1)', 'ene_cyano_F(1)', 'hzone_furan_C(1)', 'anil_no_alk_C(1)', 'hzone_acid_D(1)',
  'hzone_furan_E(1)', 'het_6_pyridone_NH2(1)', 'imine_one_fives_D(1)', 'pyrrole_M(1)',
  'pyrrole_N(1)', 'pyrrole_O(1)', 'ene_cyano_G(1)', 'sulfonamide_J(1)', 'misc_pyrrole_benz(1)',
  'thio_urea_R(1)', 'ene_one_one_B(1)', 'dhp_amino_CN_H(1)', 'het_66_anisole(1)',
  'thiazole_amine_N(1)', 'het_pyridiniums_C(1)', 'het_5_E(1)'
]

DIRNAME = os.path.join(os.path.dirname(sys.argv[0]))
PAINS_CSV = os.path.join(DIRNAME, "..", "..", "..", "Data", "Pains", "wehi_pains.csv")
PAINS_A_FILENAME = os.path.join(DIRNAME, "pains_a.in")
PAINS_B_FILENAME = os.path.join(DIRNAME, "pains_b.in")
PAINS_C_FILENAME = os.path.join(DIRNAME, "pains_c.in")

for fn in (PAINS_CSV, PAINS_A_FILENAME, PAINS_B_FILENAME, PAINS_B_FILENAME):
  if not os.path.exists(fn):
    raise IOError("Could not find necessary file: %s", fn)

sa = set(pains_a)
sb = set(pains_b)
sc = set(pains_c)

PAINS = {}

with open(PAINS_CSV) as fh:
  for smiles, name in csv.reader(fh):
    name = name.replace("<regId=", "").replace(">", "")
    PAINS[name] = smiles

PAINS_A = []
for n in pains_a:
  PAINS_A.append((n, PAINS[n]))
  del PAINS[n]
PAINS_B = []
for n in pains_b:
  PAINS_B.append((n, PAINS[n]))
  del PAINS[n]
PAINS_C = []
for n in pains_c:
  PAINS_C.append((n, PAINS[n]))
  del PAINS[n]
assert not PAINS

PAINS_A = ['{"%s","%s",0,""}' % (name, smiles) for name, smiles in PAINS_A]
PAINS_A = "const FilterData_t PAINS_A[] = {\n%s\n};" % ",\n".join(PAINS_A)

PAINS_B = ['{"%s","%s",0,""}' % (name, smiles) for name, smiles in PAINS_B]
PAINS_B = "const FilterData_t PAINS_B[] = {\n%s\n};" % ",\n".join(PAINS_B)

PAINS_C = ['{"%s","%s",0,""}' % (name, smiles) for name, smiles in PAINS_C]
PAINS_C = "const FilterData_t PAINS_C[] = {\n%s\n};" % ",\n".join(PAINS_C)


def write_pains(filename, data):
  with open(filename) as fh:
    t = fh.read()
  if t != data:
    if py3:
      import io

      # newline = don't convert to windows style
      with io.open(filename, 'w', newline='') as f:
        f.write(data)
    else:
      # wb = means don't convert newline to windows style
      with open(filename, 'wb') as f:
        f.write(data)


write_pains(PAINS_A_FILENAME, PAINS_A)
write_pains(PAINS_B_FILENAME, PAINS_B)
write_pains(PAINS_C_FILENAME, PAINS_C)

print("== Done updating pains files")



# Copyright (C) 2007-2024 Greg Landrum and other RDKit contributors
#  All Rights Reserved
#
import unittest

from rdkit.ML.Cluster import Butina


class TestCase(unittest.TestCase):

  def test1(self):
    dists = [1, 2, 1, 4, 3, 2, 6, 5, 4, 2, 7, 6, 5, 3, 1]
    nPts = 6
    cs = Butina.ClusterData(dists, nPts, 1.1, isDistData=1)
    self.assertTrue(len(cs) == 3)

    self.assertTrue(cs[0] == (1, 0, 2))
    self.assertTrue(cs[1] == (5, 4))
    self.assertTrue(cs[2] == (3, ))

  def test2(self):
    dists = [
      .5,
      1,
      .5,
      2,
      1.5,
      1,
      3,
      2.5,
      2,
      1,
      5,
      4.5,
      4,
      3,
      2,
      8,
      7.5,
      7,
      6,
      5,
      3,
      9,
      8.5,
      8,
      7,
      6,
      4,
      1,
    ]
    nPts = 8
    cs = Butina.ClusterData(dists, nPts, 2.1, isDistData=1)

    self.assertTrue(len(cs) == 3)

    self.assertTrue(cs[0] == (3, 0, 1, 2, 4))
    self.assertTrue(cs[1] == (7, 6))
    self.assertTrue(cs[2] == (5, ))

  def test3_singletons(self):
    # " edge case: everything a singleton "
    dists = [
      1,
      2,
      1,
    ]
    nPts = 3
    cs = Butina.ClusterData(dists, nPts, 0.9, isDistData=1)
    self.assertTrue(len(cs) == 3)

    self.assertTrue(cs[0] == (2, ))
    self.assertTrue(cs[1] == (1, ))
    self.assertTrue(cs[2] == (0, ))

  def test4_one_cluster(self):
    # " edge case: everything in one cluster "
    dists = [
      1,
      2,
      1,
      3,
      2,
      1,
    ]
    nPts = 4
    cs = Butina.ClusterData(dists, nPts, 2, isDistData=1)
    self.assertTrue(len(cs) == 1)
    self.assertEqual(cs[0], (2, 0, 1, 3))

  def test4b_middle_leaves(self):
    # " edge case: one in the middle leaves the edges lonely "
    dists = [
      1.5,
      2.5,
      1,
      3.5,
      2,
      1,
      5,
      3.5,
      2.5,
      1.5,
    ]
    nPts = 5
    cs = Butina.ClusterData(dists, nPts, 1.1, isDistData=1)
    self.assertTrue(len(cs) == 3)
    self.assertTrue(cs[0] == (2, 1, 3))
    self.assertTrue(cs[1] == (4, ))
    self.assertTrue(cs[2] == (0, ))

  def test6_zero_distances(self):
    # " edge case: zero distances: "
    dists = [
      1,
      2,
      0,
      2,
      0,
      0,
      4,
      2,
      2,
      2,
    ]
    nPts = 5
    cs = Butina.ClusterData(dists, nPts, 0.9, isDistData=1)
    self.assertTrue(len(cs) == 3)
    self.assertTrue(cs[0] == (3, 1, 2))
    self.assertTrue(cs[1] == (4, ))
    self.assertTrue(cs[2] == (0, ))

  def test7_reordering_nochanges(self):
    # " reordering: no changes "
    dists = [1, 2, 1, 4, 3, 2, 6, 5, 4, 2, 7, 6, 5, 3, 1]
    nPts = 6
    cs = Butina.ClusterData(dists, nPts, 1.1, isDistData=1, reordering=True)
    self.assertTrue(len(cs) == 3)

    self.assertTrue(cs[0] == (1, 0, 2))
    self.assertTrue(cs[1] == (5, 4))
    self.assertTrue(cs[2] == (3, ))

  def test8_reordering_changes(self):
    # " reordering: changes"
    dists = [
      2,
      3.5,
      1.5,
      5,
      3,
      1.5,
      7,
      5,
      3.5,
      2,
      8,
      6,
      4.5,
      3,
      1,
      9,
      7,
      5.5,
      4,
      2,
      1,
    ]
    nPts = 7
    # without reordering
    cs = Butina.ClusterData(dists, nPts, 2.1, isDistData=1)
    self.assertTrue(len(cs) == 3)
    self.assertTrue(cs[0] == (4, 3, 5, 6))
    self.assertTrue(cs[1] == (2, 1))
    self.assertTrue(cs[2] == (0, ))

    # with reordering
    cs = Butina.ClusterData(dists, nPts, 2.1, isDistData=1, reordering=True)
    self.assertTrue(len(cs) == 2)
    self.assertTrue(cs[0] == (4, 3, 5, 6))
    self.assertTrue(cs[1] == (1, 0, 2))
  
  def test9_empty_input(self):
    # " edge case: empty input "
    cs = Butina.ClusterData([], 0, 2, isDistData=1, reordering=True)
    self.assertTrue(len(cs) == 0)

  def test10_error_messages(self):
    # " error case: mismatched data dimension and nPts "
    with self.assertRaises(ValueError) as cm:
      _ = Butina.ClusterData([1, 2, 3], 1, 2, isDistData=1, reordering=True)
    self.assertEqual(str(cm.exception), "Mismatched input data dimension and nPts")
    
    # " error case: invalid input data type "
    with self.assertRaises(TypeError) as cm:
      _ = Butina.ClusterData(0, 1, 2, isDistData=1, reordering=True)
    self.assertEqual(str(cm.exception), f"Unsupported type for data, {type(0)}")

profileTest = 0

if __name__ == '__main__':  # pragma: nocover
  unittest.main()
